/* This is the header file of the Parma Polyhedra Library.
   Copyright (C) 2001, 2002 Roberto Bagnara <bagnara@cs.unipr.it>

This file is part of the Parma Polyhedra Library (PPL).

The PPL is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

The PPL is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

For the most up-to-date information see the Parma Polyhedra Library
site: http://www.cs.unipr.it/ppl/ . */

#ifndef _ppl_hh
#define _ppl_hh 1

#ifdef NDEBUG
#define PPL_SAVE_NDEBUG 1
#undef NDEBUG
#endif

/* config.h.  Generated by configure.  */
/* config.h.in.  Generated from configure.ac by autoheader.  */


#ifndef _ppl_config_h
#define _ppl_config_h 1


/* Not zero if GMP has been compiled with support for exceptions. */
#define GMP_SUPPORTS_EXCEPTIONS 0

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/resource.h> header file. */
#define HAVE_SYS_RESOURCE_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Assertions are disabled when this is defined */
#define NDEBUG 1

/* Name of package */
#define PACKAGE "ppl"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "ppl-devel@cs.unipr.it"

/* Define to the full name of this package. */
#define PACKAGE_NAME "the Parma Polyhedra Library"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "the Parma Polyhedra Library 0.4.2"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "ppl"

/* Define to the version of this package. */
#define PACKAGE_VERSION "0.4.2"

/* The size of a `mp_limb_t', as computed by sizeof. */
#define SIZEOF_MP_LIMB_T 0

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Version number of package */
#define VERSION "0.4.2"

/* Define to empty if `const' does not conform to ANSI C. */
/* #undef const */

/* Define as `__inline' if that's what the C compiler calls it, or to nothing
   if it is not supported. */
/* #undef inline */



// Neutralize relops for GCC 2.96.
#ifndef __SGI_STL_INTERNAL_RELOPS
#define __SGI_STL_INTERNAL_RELOPS
#endif

#if __GNUC__ >= 3
// Flexible array members at the end of a struct
// are defined as in
//   Type array[];
#define PPL_FLEXIBLE_ARRAY
#else
// Flexible array members at the end of a struct
// are defined as in
//   Type array[0];
#define PPL_FLEXIBLE_ARRAY 0
#endif

#endif


#if __GNUC__ < 3
// Get around a bug in GCC versions prior to 3 whereby the <cassert>
// header is wrongly guarded against multiple inclusion.
#ifdef __CASSERT__
#undef __CASSERT__
#endif

#endif

#include <algorithm>
#include <cassert>
#include <cstddef>
#include <exception>
#include <gmp.h>
#include <gmpxx.h>
#include <iosfwd>
#include <iterator>
#include <set>
#include <vector>
namespace Parma_Polyhedra_Library {

  typedef mpz_class Integer;
}
namespace Parma_Polyhedra_Library {
  class ExtendedRational;
  class Boundary;
  class LBoundary;
  class UBoundary;
  class Interval;
}
namespace Parma_Polyhedra_Library {
  class BoundingBox;
}
namespace Parma_Polyhedra_Library {
  class Constraint;
}
namespace Parma_Polyhedra_Library {
  class Generator;
}
namespace Parma_Polyhedra_Library {
  class Init;
}
namespace Parma_Polyhedra_Library {
  class SatRow;
}
namespace Parma_Polyhedra_Library {
  class SatMatrix;
}
namespace Parma_Polyhedra_Library {
  class Row;
}
namespace Parma_Polyhedra_Library {
  class Matrix;
}
namespace Parma_Polyhedra_Library {
  class Variable;
}
namespace Parma_Polyhedra_Library {
  class LinExpression;
}
namespace Parma_Polyhedra_Library {
  class ConSys;
}
namespace Parma_Polyhedra_Library {
  class GenSys;
}
namespace Parma_Polyhedra_Library {
  class Status;
}
namespace Parma_Polyhedra_Library {
  class Poly_Con_Relation;
}
namespace Parma_Polyhedra_Library {
  class Poly_Gen_Relation;
}
namespace Parma_Polyhedra_Library {
  class Polyhedron;
}
namespace std {
}
namespace Parma_Polyhedra_Library {
  class C_Polyhedron;
}
namespace Parma_Polyhedra_Library {
  class NNC_Polyhedron;
}
namespace Parma_Polyhedra_Library {

  inline void negate(Integer& x);
  inline void gcd_assign(Integer& x, const Integer& y);
  inline void gcd_assign(Integer& x, const Integer& y, const Integer& z);
  inline void exact_div_assign(Integer& x, const Integer& y);
  inline void exact_div_assign(Integer& x, const Integer& y, const Integer& z);
  inline int cmp(const Integer& x, const Integer& y);
  inline const Integer& Integer_zero();
  inline const Integer& Integer_one();

}

namespace std {


void swap(Parma_Polyhedra_Library::Integer& x,
          Parma_Polyhedra_Library::Integer& y);

}
namespace Parma_Polyhedra_Library {

inline void
negate(Integer& x) {
  mpz_neg(x.get_mpz_t(), x.get_mpz_t());
}

inline void
gcd_assign(Integer& x, const Integer& y) {
  mpz_gcd(x.get_mpz_t(), x.get_mpz_t(), y.get_mpz_t());
}

inline void
gcd_assign(Integer& x, const Integer& y, const Integer& z) {
  mpz_gcd(x.get_mpz_t(), y.get_mpz_t(), z.get_mpz_t());
}

inline void
exact_div_assign(Integer& x, const Integer& y) {
  mpz_divexact(x.get_mpz_t(), x.get_mpz_t(), y.get_mpz_t());
}

inline void
exact_div_assign(Integer& x, const Integer& y, const Integer& z) {
  mpz_divexact(x.get_mpz_t(), y.get_mpz_t(), z.get_mpz_t());
}

inline int
cmp(const Integer& x, const Integer& y) {
  return mpz_cmp(x.get_mpz_t(), y.get_mpz_t());
}

inline const Integer&
Integer_zero() {
  static Integer z(0);
  return z;
}

inline const Integer&
Integer_one() {
  static Integer o(1);
  return o;
}

}

inline void
std::swap(Parma_Polyhedra_Library::Integer& x,
          Parma_Polyhedra_Library::Integer& y) {
  mpz_swap(x.get_mpz_t(), y.get_mpz_t());
}
namespace Parma_Polyhedra_Library {

  bool operator==(const ExtendedRational& x, const ExtendedRational& y);
  bool operator!=(const ExtendedRational& x, const ExtendedRational& y);
  bool operator< (const ExtendedRational& x, const ExtendedRational& y);
  bool operator> (const ExtendedRational& x, const ExtendedRational& y);
  bool operator<=(const ExtendedRational& x, const ExtendedRational& y);
  bool operator>=(const ExtendedRational& x, const ExtendedRational& y);
}

class Parma_Polyhedra_Library::ExtendedRational {
private:

  int e;

  mpq_class v;

public:
  ExtendedRational(const mpq_class& n);

  ExtendedRational(const Integer& num, const Integer& den);

  explicit ExtendedRational(char sign);

  ExtendedRational(const ExtendedRational& y);

  ExtendedRational& operator=(const ExtendedRational& y);

  int direction_of_infinity() const;






  void canonicalize() const;

  Integer numerator() const;

  Integer denominator() const;


  friend bool
  Parma_Polyhedra_Library::operator==(const ExtendedRational& x,
                                      const ExtendedRational& y);
  friend bool
  Parma_Polyhedra_Library::operator!=(const ExtendedRational& x,
                                      const ExtendedRational& y);
  friend bool
  Parma_Polyhedra_Library::operator<(const ExtendedRational& x,
                                     const ExtendedRational& y);
  friend bool
  Parma_Polyhedra_Library::operator>(const ExtendedRational& x,
                                     const ExtendedRational& y);
  friend bool
  Parma_Polyhedra_Library::operator<=(const ExtendedRational& x,
                                      const ExtendedRational& y);
  friend bool
  Parma_Polyhedra_Library::operator>=(const ExtendedRational& x,
                                      const ExtendedRational& y);
};

namespace Parma_Polyhedra_Library {

  bool operator==(const Boundary& x, const Boundary& y);
  bool operator!=(const Boundary& x, const Boundary& y);
  bool operator< (const Boundary& x, const Boundary& y);
  bool operator> (const Boundary& x, const Boundary& y);
  bool operator<=(const Boundary& x, const Boundary& y);
  bool operator>=(const Boundary& x, const Boundary& y);

  bool operator==(const Boundary& x, ExtendedRational y);
  bool operator!=(const Boundary& x, ExtendedRational y);
  bool operator< (const Boundary& x, ExtendedRational y);
  bool operator> (const Boundary& x, ExtendedRational y);
  bool operator<=(const Boundary& x, ExtendedRational y);
  bool operator>=(const Boundary& x, ExtendedRational y);

  bool operator< (const LBoundary& x, const UBoundary& y);
  bool operator> (const UBoundary& x, const LBoundary& y);
  bool operator<=(const UBoundary& x, const LBoundary& y);
  bool operator>=(const LBoundary& x, const UBoundary& y);

  bool operator< (const LBoundary& x, ExtendedRational y);
  bool operator> (const UBoundary& x, ExtendedRational y);
  bool operator<=(const UBoundary& x, ExtendedRational y);
  bool operator>=(const LBoundary& x, ExtendedRational y);
}

class Parma_Polyhedra_Library::Boundary {
protected:
  enum Flag { NEG = -1, ZERO = 0, POS = 1 };

  ExtendedRational value;
  Flag flag;

  Boundary(const ExtendedRational& v, Flag f);

  friend bool
  Parma_Polyhedra_Library::operator==(const Boundary& x, const Boundary& y);
  friend bool
  Parma_Polyhedra_Library::operator!=(const Boundary& x, const Boundary& y);
  friend bool
  Parma_Polyhedra_Library::operator<(const Boundary& x, const Boundary& y);
  friend bool
  Parma_Polyhedra_Library::operator>(const Boundary& x, const Boundary& y);
  friend bool
  Parma_Polyhedra_Library::operator<=(const Boundary& x, const Boundary& y);
  friend bool
  Parma_Polyhedra_Library::operator>=(const Boundary& x, const Boundary& y);

  friend bool
  Parma_Polyhedra_Library::operator==(const Boundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator!=(const Boundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator<(const Boundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator>(const Boundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator<=(const Boundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator>=(const Boundary& x, ExtendedRational y);

  friend bool
  Parma_Polyhedra_Library::operator< (const LBoundary& x, const UBoundary& y);
  friend bool
  Parma_Polyhedra_Library::operator> (const UBoundary& x, const LBoundary& y);
  friend bool
  Parma_Polyhedra_Library::operator<=(const UBoundary& x, const LBoundary& y);
  friend bool
  Parma_Polyhedra_Library::operator>=(const LBoundary& x, const UBoundary& y);

  friend bool
  Parma_Polyhedra_Library::operator< (const LBoundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator> (const UBoundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator<=(const UBoundary& x, ExtendedRational y);
  friend bool
  Parma_Polyhedra_Library::operator>=(const LBoundary& x, ExtendedRational y);

public:
  bool is_closed() const;

  const ExtendedRational& bound() const;
};


class Parma_Polyhedra_Library::LBoundary : public Boundary {
public:

  enum OpenClosed { OPEN = Boundary::POS, CLOSED = Boundary::ZERO };

  LBoundary();
  LBoundary(const ExtendedRational& v, OpenClosed f);

  bool OK() const;
};

class Parma_Polyhedra_Library::UBoundary : public Boundary {
public:

  enum OpenClosed { OPEN = Boundary::NEG, CLOSED = Boundary::ZERO };

  UBoundary();
  UBoundary(const ExtendedRational& v, OpenClosed f);

  bool OK() const;
};

class Parma_Polyhedra_Library::Interval {
public:

  Interval();

  bool is_empty() const;

  const LBoundary& lower_bound() const;

  const UBoundary& upper_bound() const;

  void raise_lower_bound(LBoundary new_lower);

  void lower_upper_bound(UBoundary new_upper);

  void set_empty();

  bool OK() const;

private:
  LBoundary lower;
  UBoundary upper;
};
namespace Parma_Polyhedra_Library {

inline
ExtendedRational::ExtendedRational(const mpq_class& n)
  : e(0), v(n) {
}

inline
ExtendedRational::ExtendedRational(const Integer& num, const Integer& den)
  : e(0), v(num, den) {
}

inline
ExtendedRational::ExtendedRational(char sign)
  : e(sign == '+' ? 1 : -1) {
  assert(sign == '+' || sign == '-');
}

inline
ExtendedRational::ExtendedRational(const ExtendedRational& y)
  : e(y.e) {
  if (e == 0)
    v = y.v;
}

inline ExtendedRational&
ExtendedRational::operator=(const ExtendedRational& y) {
  e = y.e;
  if (e == 0)
    v = y.v;
  return *this;
}

inline int
ExtendedRational::direction_of_infinity() const {
  return e;
}
inline void
ExtendedRational::canonicalize() const {
  const_cast<ExtendedRational*>(this)->v.canonicalize();
}

inline Integer
ExtendedRational::numerator() const {
  canonicalize();
  return v.get_num();
}

inline Integer
ExtendedRational::denominator() const {
  canonicalize();
  return v.get_den();
}


inline bool
operator==(const ExtendedRational& x, const ExtendedRational& y) {
  return x.e == y.e && (x.e != 0 || x.v == y.v);
}

inline bool
operator!=(const ExtendedRational& x, const ExtendedRational& y) {
  return !(x == y);
}

inline bool
operator<(const ExtendedRational& x, const ExtendedRational& y) {
  return x.e < y.e || (x.e == 0 && y.e == 0 && x.v < y.v);
}

inline bool
operator>(const ExtendedRational& x, const ExtendedRational& y) {
  return y < x;
}

inline bool
operator<=(const ExtendedRational& x, const ExtendedRational& y) {
  return x < y || x == y;
}

inline bool
operator>=(const ExtendedRational& x, const ExtendedRational& y) {
  return y <= x;
}

inline
Boundary::Boundary(const ExtendedRational& v, Flag f)
  : value(v), flag(f) {
}

inline bool
Boundary::is_closed() const {
  return flag == ZERO;
}

inline const ExtendedRational&
Boundary::bound() const {
  return value;
}

inline
LBoundary::LBoundary()
  : Boundary(ExtendedRational('-'), POS) {
}

inline
LBoundary::LBoundary(const ExtendedRational& v, OpenClosed f)
  : Boundary(v, f == CLOSED ? ZERO : POS) {
}

inline
UBoundary::UBoundary()
  : Boundary(ExtendedRational('+'), NEG) {
}

inline
UBoundary::UBoundary(const ExtendedRational& v, OpenClosed f)
  : Boundary(v, f == CLOSED ? ZERO : NEG) {
}

inline bool
operator==(const Boundary& x, const Boundary& y) {
  return x.value == y.value && x.flag == y.flag;
}

inline bool
operator!=(const Boundary& x, const Boundary& y) {
  return !(x == y);
}

inline bool
operator<(const Boundary& x, const Boundary& y) {
  return x.value < y.value ||
    (x.value == y.value && x.flag < y.flag);
}

inline bool
operator<(const LBoundary& x, const UBoundary& y) {
  return x.value < y.value;
}

inline bool
operator>(const Boundary& x, const Boundary& y) {
  return x.value > y.value ||
    (x.value == y.value && x.flag > y.flag);
}

inline bool
operator>(const UBoundary& x, const LBoundary& y) {
  return x.value > y.value;
}

inline bool
operator<=(const Boundary& x, const Boundary& y) {
  return !(x > y);
}

inline bool
operator<=(const UBoundary& x, const LBoundary& y) {
  return !(x > y);
}

inline bool
operator>=(const Boundary& x, const Boundary& y) {
  return !(x < y);
}

inline bool
operator>=(const LBoundary& x, const UBoundary& y) {
  return !(x < y);
}

inline bool
operator==(const Boundary& x, ExtendedRational y) {
  return x.value == y && x.flag == Boundary::ZERO;
}

inline bool
operator!=(const Boundary& x, ExtendedRational y) {
  return !(x == y);
}

inline bool
operator<(const Boundary& x, ExtendedRational y) {
  return x.value < y ||
    (x.value == y && x.flag < Boundary::ZERO);
}

inline bool
operator<(const LBoundary& x, ExtendedRational y) {
  return x.value < y;
}

inline bool
operator>(const Boundary& x, ExtendedRational y) {
  return x.value > y ||
    (x.value == y && x.flag > Boundary::ZERO);
}

inline bool
operator>(const UBoundary& x, ExtendedRational y) {
  return x.value > y;
}

inline bool
operator<=(const Boundary& x, ExtendedRational y) {
  return !(x > y);
}

inline bool
operator<=(const UBoundary& x, ExtendedRational y) {
  return !(x > y);
}

inline bool
operator>=(const Boundary& x, ExtendedRational y) {
  return !(x < y);
}

inline bool
operator>=(const LBoundary& x, ExtendedRational y) {
  return !(x < y);
}

inline
Interval::Interval() {
}

inline bool
Interval::is_empty() const {
  return lower > upper;
}

inline const LBoundary&
Interval::lower_bound() const {
  return lower;
}

inline const UBoundary&
Interval::upper_bound() const {
  return upper;
}

inline void
Interval::raise_lower_bound(LBoundary new_lower) {
  if (new_lower > lower)
    lower = new_lower;
}

inline void
Interval::lower_upper_bound(UBoundary new_upper) {
  if (new_upper < upper)
    upper = new_upper;
}

inline void
Interval::set_empty() {
  lower = LBoundary(ExtendedRational('+'), LBoundary::OPEN);
  upper = UBoundary(ExtendedRational('-'), UBoundary::OPEN);
  assert(is_empty());
}

}
namespace Parma_Polyhedra_Library {

class BoundingBox {
public:

  BoundingBox(unsigned int num_dimensions);


  unsigned int space_dimension() const;



  const Interval& operator[](size_t k) const;


  bool is_empty() const;
  bool get_lower_bound(unsigned int k, bool& closed,
                       Integer& n, Integer& d) const;
  bool get_upper_bound(unsigned int k, bool& closed,
                       Integer& n, Integer& d) const;


  void set_empty();
  void raise_lower_bound(unsigned int k, bool closed,
                         const Integer& n, const Integer& d);
  void lower_upper_bound(unsigned int k, bool closed,
                         const Integer& n, const Integer& d);

private:
  std::vector<Interval> vec;
  mutable bool empty;
  mutable bool empty_up_to_date;
};

std::ostream& operator<<(std::ostream& s, const BoundingBox& bbox);

}
namespace Parma_Polyhedra_Library {

inline
BoundingBox::BoundingBox(unsigned int num_dimensions)
  : vec(num_dimensions), empty(false), empty_up_to_date(true) {
}

inline unsigned int
BoundingBox::space_dimension() const {
  return vec.size();
}

inline const Interval&
BoundingBox::operator[](size_t k) const {
  return vec[k];
}

inline bool
BoundingBox::is_empty() const {
  if (empty_up_to_date)
    return empty;
  else {
    empty_up_to_date = true;
    for (size_t k = vec.size(); k-- > 0; )
      if (vec[k].is_empty()) {
        empty = true;
        return true;
      }
    empty = false;
    return false;
  }
}

inline bool
BoundingBox::get_lower_bound(unsigned int k, bool& closed,
                             Integer& n, Integer& d) const {
  const LBoundary& lb = vec[k].lower_bound();
  const ExtendedRational& lr = lb.bound();

  if (lr.direction_of_infinity() != 0)
    return false;

  closed = lb.is_closed();
  n = lr.numerator();
  d = lr.denominator();

  return true;
}

inline bool
BoundingBox::get_upper_bound(unsigned int k, bool& closed,
                             Integer& n, Integer& d) const {
  const UBoundary& ub = vec[k].upper_bound();
  const ExtendedRational& ur = ub.bound();

  if (ur.direction_of_infinity() != 0)
    return false;

  closed = ub.is_closed();
  n = ur.numerator();
  d = ur.denominator();
  return true;
}

inline void
BoundingBox::set_empty() {
  empty = empty_up_to_date = true;
}

inline void
BoundingBox::raise_lower_bound(unsigned int k, bool closed,
                               const Integer& n, const Integer& d) {
  vec[k].raise_lower_bound(LBoundary(ExtendedRational(n, d),
                                     (closed
                                      ? LBoundary::CLOSED
                                      : LBoundary::OPEN)));
  empty_up_to_date = false;
}

inline void
BoundingBox::lower_upper_bound(unsigned int k, bool closed,
                               const Integer& n, const Integer& d) {
  vec[k].lower_upper_bound(UBoundary(ExtendedRational(n, d),
                                     (closed
                                      ? UBoundary::CLOSED
                                      : UBoundary::OPEN)));
  empty_up_to_date = false;
}

}
namespace Parma_Polyhedra_Library {




extern Integer* tmp_Integer;
inline size_t
compute_capacity(size_t requested_size) {
  return 2*(requested_size + 1);
}







class Throwable {
public:

  virtual void throw_me() const = 0;
};
extern const Throwable* volatile abandon_exponential_computations;







inline void
maybe_abandon() {
  if (const Throwable* p = abandon_exponential_computations)
    p->throw_me();
}





struct From_Bounding_Box {
};





  const Integer& operator*(const Constraint& c, const Generator& g);






  const Integer& reduced_scalar_product(const Constraint& c,
                                        const Generator& g);

}
class Parma_Polyhedra_Library::Init {
private:

  static unsigned count;

public:

  Init();


  ~Init();
};
namespace Parma_Polyhedra_Library {

inline
Init::Init() {


  if (count++ == 0)
    tmp_Integer = new Integer[6];
}

inline
Init::~Init() {


  if (--count == 0)
    delete[] tmp_Integer;
}

}
static Parma_Polyhedra_Library::Init Parma_Polyhedra_Library__initializer;
namespace Parma_Polyhedra_Library {




  enum Topology {
    NECESSARILY_CLOSED = 0,
    NOT_NECESSARILY_CLOSED = 1
  };

}
namespace Parma_Polyhedra_Library {

  int compare(const SatRow& x, const SatRow& y);
  bool operator==(const SatRow& x, const SatRow& y);
  bool operator!=(const SatRow& x, const SatRow& y);
  bool operator <(const SatRow& x, const SatRow& y);
  bool operator >(const SatRow& x, const SatRow& y);
  bool operator<=(const SatRow& x, const SatRow& y);
  bool operator>=(const SatRow& x, const SatRow& y);
  void set_union(const SatRow& x, const SatRow& y, SatRow& z);
}

class Parma_Polyhedra_Library::SatRow {
private:

  mpz_t vec;

  static unsigned int last_one(mp_limb_t w);
  static unsigned int first_one(mp_limb_t w);

public:

  SatRow();


  SatRow(const SatRow& y);


  ~SatRow();


  SatRow& operator=(const SatRow& y);


  void swap(SatRow& y);


  bool operator[](size_t k) const;


  void set(size_t k);


  void clear(size_t k);


  void clear_from(size_t k);


  void clear();


  friend int
  Parma_Polyhedra_Library::compare(const SatRow& x,
                                   const SatRow& y);



  friend bool
  Parma_Polyhedra_Library::operator==(const SatRow& x,
                                      const SatRow& y);
  friend bool
  Parma_Polyhedra_Library::operator!=(const SatRow& x,
                                      const SatRow& y);
  friend bool
  Parma_Polyhedra_Library::operator <(const SatRow& x,
                                      const SatRow& y);
  friend bool
  Parma_Polyhedra_Library::operator >(const SatRow& x,
                                      const SatRow& y);
  friend bool
  Parma_Polyhedra_Library::operator<=(const SatRow& x,
                                      const SatRow& y);
  friend bool
  Parma_Polyhedra_Library::operator>=(const SatRow& x,
                                      const SatRow& y);



  friend void Parma_Polyhedra_Library::set_union(const SatRow& x,
                                                 const SatRow& y,
                                                 SatRow& z);


  size_t size();


  int first() const;



  int next(int position) const;


  int last() const;



  int prev(int position) const;


  size_t count_ones() const;


  bool empty() const;


  bool OK() const;
};

namespace Parma_Polyhedra_Library {




  std::ostream& operator<<(std::ostream& s, const SatRow& r);
}


namespace std {





void swap(Parma_Polyhedra_Library::SatRow& x,
          Parma_Polyhedra_Library::SatRow& y);

}
namespace Parma_Polyhedra_Library {

inline
SatRow::SatRow() {
  mpz_init(vec);
}

inline
SatRow::SatRow(const SatRow& y) {
  mpz_init_set(vec, y.vec);
}

inline
SatRow::~SatRow() {
  mpz_clear(vec);
}

inline SatRow&
SatRow::operator=(const SatRow& y) {
  mpz_set(vec, y.vec);
  return *this;
}

inline bool
SatRow::operator[](size_t k) const {
  return mpz_tstbit(vec, k);
}

inline void
SatRow::set(size_t k) {
  mpz_setbit(vec, k);
}

inline void
SatRow::clear(size_t k) {
  mpz_clrbit(vec, k);
}

inline void
SatRow::clear_from(size_t k) {
  mpz_tdiv_r_2exp(vec, vec, k);
}




inline size_t
SatRow::count_ones() const {
  return mpz_popcount(vec);
}




inline bool
SatRow::empty() const {
  return mpz_sgn(vec) == 0;
}




inline void
SatRow::swap(SatRow& y) {
  mpz_swap(vec, y.vec);
}




inline void
SatRow::clear() {
  mpz_set_ui(vec, 0UL);
}

inline bool
operator==(const SatRow& x, const SatRow& y) {
  return mpz_cmp(x.vec, y.vec) == 0;
}

inline bool
operator!=(const SatRow& x, const SatRow& y) {
  return mpz_cmp(x.vec, y.vec) != 0;
}

inline bool
operator>(const SatRow& x, const SatRow& y) {
  return y < x;
}

inline bool
operator>=(const SatRow& x, const SatRow& y) {
  return y <= x;
}

inline void
set_union(const SatRow& x, const SatRow& y, SatRow& z) {
  mpz_ior(z.vec, x.vec, y.vec);
}

}


namespace std {


inline void
swap(Parma_Polyhedra_Library::SatRow& x, Parma_Polyhedra_Library::SatRow& y) {
  x.swap(y);
}
inline void
iter_swap(std::vector<Parma_Polyhedra_Library::SatRow>::iterator x,
          std::vector<Parma_Polyhedra_Library::SatRow>::iterator y) {
  swap(*x, *y);
}

}
class Parma_Polyhedra_Library::SatMatrix {
private:

  std::vector<SatRow> rows;


  size_t row_size;

  struct RowCompare {
    bool operator()(const SatRow& x, const SatRow& y) const;
  };

public:

  SatMatrix();



  SatMatrix(size_t n_rows, size_t n_columns);


  SatMatrix(const SatMatrix& y);


  ~SatMatrix();


  SatMatrix& operator=(const SatMatrix& y);


  void swap(SatMatrix& y);


  SatRow& operator[](size_t k);


  const SatRow& operator[](size_t k) const;


  void clear();


  void transpose();


  void transpose_assign(const SatMatrix& y);


  size_t num_columns() const;


  size_t num_rows() const;


  void sort_rows();


  bool sorted_contains(const SatRow& row) const;


  void add_row(const SatRow& row);


  void rows_erase_to_end(size_t first_to_erase);


  void columns_erase_to_end(size_t first_to_erase);


  void resize(size_t new_n_rows, size_t new_n_columns);


  bool OK() const;






};

namespace std {





void swap(Parma_Polyhedra_Library::SatMatrix& x,
          Parma_Polyhedra_Library::SatMatrix& y);

}

namespace Parma_Polyhedra_Library {





  bool operator==(const SatMatrix& x, const SatMatrix& y);






  bool operator!=(const SatMatrix& x, const SatMatrix& y);





  std::ostream& operator<<(std::ostream& s, const SatMatrix& x);





  std::istream& operator>>(std::istream& s, SatMatrix& x);
}
namespace Parma_Polyhedra_Library {

inline
SatMatrix::SatMatrix()
  : rows(),
    row_size(0) {
}

inline
SatMatrix::SatMatrix(size_t n_rows, size_t n_columns)
  : rows(n_rows),
    row_size(n_columns) {
}

inline
SatMatrix::SatMatrix(const SatMatrix& y)
  : rows(y.rows),
    row_size(y.row_size) {
}

inline
SatMatrix::~SatMatrix() {
}




inline void
SatMatrix::rows_erase_to_end(size_t first_to_erase) {


  assert(first_to_erase <= rows.size());
  if (first_to_erase < rows.size())
    rows.erase(rows.begin() + first_to_erase, rows.end());
  assert(OK());
}




inline void
SatMatrix::columns_erase_to_end(size_t first_to_erase) {


  assert(first_to_erase <= row_size);
  row_size = first_to_erase;
  assert(OK());
}




inline void
SatMatrix::swap(SatMatrix& y) {
  std::swap(row_size, y.row_size);
  std::swap(rows, y.rows);
}




inline SatRow&
SatMatrix::operator[](size_t k) {
  assert(k < rows.size());
  return rows[k];
}




inline const SatRow&
SatMatrix::operator[](size_t k) const {
  assert(k < rows.size());
  return rows[k];
}

inline size_t
SatMatrix::num_columns() const {
  return row_size;
}

inline size_t
SatMatrix::num_rows() const {
  return rows.size();
}

inline void
SatMatrix::clear() {

  std::vector<SatRow>().swap(rows);
  row_size = 0;
}


inline bool
operator!=(const SatMatrix& x, const SatMatrix& y) {
  return !(x == y);
}

inline bool
SatMatrix::RowCompare::
operator()(const SatRow& x, const SatRow& y) const {
  return compare(x, y) < 0;
}

}


namespace std {


inline void
swap(Parma_Polyhedra_Library::SatMatrix& x,
     Parma_Polyhedra_Library::SatMatrix& y) {
  x.swap(y);
}

}
namespace Parma_Polyhedra_Library {

  std::ostream& operator<<(std::ostream& s, const Row& row);
}
class Parma_Polyhedra_Library::Row {
public:

  const Integer& inhomogeneous_term() const;


  const Integer& coefficient(size_t n) const;

  enum Kind {
    LINE_OR_EQUALITY = 0,
    RAY_OR_POINT_OR_INEQUALITY = 1
  };


  class Type;


  Row(Type t, size_t sz);


  Row(Type t, size_t sz, size_t capacity);



  void construct(Type t, size_t sz);
  void construct(Type t, size_t sz, size_t capacity);



  Row();


  Row(const Row& y);


  Row(const Row& y, size_t capacity);


  Row(const Row& y, size_t sz, size_t capacity);


  ~Row();


  Row& operator=(const Row& y);


  void swap(Row& y);


  void assign(Row& y);


  void resize_no_copy(size_t new_size);


  void grow_no_copy(size_t new_size);


  void shrink(size_t new_size);



  Integer& operator[](size_t k);
  const Integer& operator[](size_t k) const;




  Type type() const;
  Topology topology() const;
  bool is_line_or_equality() const;
  bool is_ray_or_point_or_inequality() const;
  bool is_necessarily_closed() const;




  void set_is_line_or_equality();
  void set_is_ray_or_point_or_inequality();
  void set_necessarily_closed();
  void set_not_necessarily_closed();



  size_t size() const;


  size_t space_dimension() const;


  void normalize();



  void strong_normalize();


  void linear_combine(const Row& y, size_t k);



  bool all_homogeneous_terms_are_zero() const;


  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s, const Row& row);


  bool OK(size_t row_size, size_t row_capacity) const;

private:
  friend class Parma_Polyhedra_Library::LinExpression;
  friend class Parma_Polyhedra_Library::Constraint;
  friend class Parma_Polyhedra_Library::Generator;

  class Impl;


  Impl* impl;
};

namespace Parma_Polyhedra_Library {





  const Integer& operator*(const Row& x, const Row& y);






  const Integer& reduced_scalar_product(const Row& x, const Row& y);





  int compare(const Row& x, const Row& y);






  bool operator==(const Row& x, const Row& y);

  bool operator!=(const Row& x, const Row& y);

  bool operator<=(const Row& x, const Row& y);

  bool operator <(const Row& x, const Row& y);

  bool operator>=(const Row& x, const Row& y);

  bool operator >(const Row& x, const Row& y);




}





class Parma_Polyhedra_Library::Row::Type {
public:

  Type();



  Type(Topology topol, Kind kind);



  Topology topology() const;
  bool is_line_or_equality() const;
  bool is_ray_or_point_or_inequality() const;
  bool is_necessarily_closed() const;

  void set_is_line_or_equality();
  void set_is_ray_or_point_or_inequality();
  void set_necessarily_closed();
  void set_not_necessarily_closed();


private:

  typedef unsigned int flags_t;


  Type(flags_t mask);


  flags_t flags;



  static const flags_t NNC = NOT_NECESSARILY_CLOSED << 0;
  static const flags_t RPI = RAY_OR_POINT_OR_INEQUALITY << 1;



  bool test_all(flags_t mask) const;


  void set(flags_t mask);


  void reset(flags_t mask);
};


class Parma_Polyhedra_Library::Row::Impl {
public:


  void* operator new(size_t fixed_size, size_t capacity);
  void operator delete(void* p, size_t capacity);
  void operator delete(void* p);



  Impl(Type t, size_t sz);


  Impl(const Impl& y);


  Impl(const Impl& y, size_t sz);


  ~Impl();


  void resize_no_copy(size_t new_size);


  void grow_no_copy(size_t new_size);


  void shrink(size_t new_size);



  Integer& operator[](size_t k);
  const Integer& operator[](size_t k) const;




  size_t size() const;
  void set_size(size_t new_sz);
  void bump_size();


private:

  size_t size_;

public:


  Type type;

private:

  Integer vec_[PPL_FLEXIBLE_ARRAY];

private:

  Impl();


  Impl& operator=(const Impl&);


  void copy_construct(const Impl& y);
};

namespace std {





void swap(Parma_Polyhedra_Library::Row& x,
          Parma_Polyhedra_Library::Row& y);

}
namespace Parma_Polyhedra_Library {






inline void*
Row::Impl::operator new(size_t fixed_size, size_t capacity) {
  return ::operator new(fixed_size + capacity*sizeof(Integer));
}






inline void
Row::Impl::operator delete(void* p) {
  ::operator delete(p);
}






inline void
Row::Impl::operator delete(void* p, size_t) {
  ::operator delete(p);
}





inline size_t
Row::Impl::size() const {
  return size_;
}





inline void
Row::Impl::set_size(size_t new_sz) {
  size_ = new_sz;
}





inline void
Row::Impl::bump_size() {
  ++size_;
}







inline void
Row::Impl::resize_no_copy(size_t new_size) {
  if (new_size < size())
    shrink(new_size);
  else
    grow_no_copy(new_size);
}


inline
Row::Impl::Impl(Type t, size_t sz)
  : size_(0), type(t) {
  grow_no_copy(sz);
}


inline
Row::Impl::Impl(const Impl& y)
  : size_(0), type(y.type) {
  copy_construct(y);
}

inline
Row::Impl::Impl(const Impl& y, size_t sz)
  : size_(0), type(y.type) {
  copy_construct(y);
  grow_no_copy(sz);
}






inline
Row::Impl::~Impl() {
  shrink(0);
}





inline Integer&
Row::Impl::operator[](size_t k) {
  assert(k < size());
  return vec_[k];
}





inline const Integer&
Row::Impl::operator[](size_t k) const {
  assert(k < size());
  return vec_[k];
}


inline
Row::Type::Type()
  : flags(0) {
}

inline
Row::Type::Type(Topology topol, Kind kind)
  : flags(static_cast<flags_t>(topol | (kind << 1))) {
}

inline
Row::Type::Type(flags_t mask)
  : flags(mask) {
}

inline bool
Row::Type::test_all(flags_t mask) const {
  return (flags & mask) == mask;
}

inline void
Row::Type::set(flags_t mask) {
  flags |= mask;
}

inline void
Row::Type::reset(flags_t mask) {
  flags &= ~mask;
}

inline bool
Row::Type::is_ray_or_point_or_inequality() const {
  return test_all(RPI);
}

inline void
Row::Type::set_is_ray_or_point_or_inequality() {
  set(RPI);
}

inline bool
Row::Type::is_line_or_equality() const {
  return !is_ray_or_point_or_inequality();
}

inline void
Row::Type::set_is_line_or_equality() {
  reset(RPI);
}

inline Topology
Row::Type::topology() const {
  return test_all(NNC) ? NOT_NECESSARILY_CLOSED : NECESSARILY_CLOSED;
}

inline bool
Row::Type::is_necessarily_closed() const {
  return !test_all(NNC);
}

inline void
Row::Type::set_necessarily_closed() {
  reset(NNC);
}

inline void
Row::Type::set_not_necessarily_closed() {
  set(NNC);
}




inline size_t
Row::size() const {
  return impl->size();
}


inline Row::Type
Row::type() const {
  return impl->type;
}






inline bool
Row::is_necessarily_closed() const {
  return type().is_necessarily_closed();
}

inline size_t
Row::space_dimension() const {
  size_t sz = size();
  return (sz == 0)
    ? 0
    : sz - (is_necessarily_closed() ? 1 : 2);
}
inline
Row::Row()
  : impl(0) {
}
inline void
Row::construct(Type t, size_t sz, size_t capacity) {
  assert(capacity >= sz);
  impl = new (capacity) Impl(t, sz);



}





inline void
Row::construct(Type t, size_t sz) {
  construct(t, sz, sz);
}

inline
Row::Row(Type t, size_t sz, size_t capacity) {
  construct(t, sz, capacity);
}

inline
Row::Row(Type t, size_t sz) {
  construct(t, sz);
}

inline
Row::Row(const Row& y)
  : impl(y.impl
         ? new (compute_capacity(y.size())) Impl(*y.impl)
         : 0) {



}





inline
Row::Row(const Row& y, size_t capacity) {
  assert(capacity >= y.size());
  impl = y.impl ? new (capacity) Impl(*y.impl) : 0;



}






inline
Row::Row(const Row& y, size_t sz, size_t capacity) {
  assert(capacity >= y.size());
  impl = y.impl ? new (capacity) Impl(*y.impl, sz) : 0;



}

inline
Row::~Row() {
  delete impl;
}





inline void
Row::resize_no_copy(size_t new_sz) {
  assert(impl);



  impl->resize_no_copy(new_sz);
}





inline void
Row::grow_no_copy(size_t new_sz) {
  assert(impl);



  impl->grow_no_copy(new_sz);
}





inline void
Row::shrink(size_t new_sz) {
  assert(impl);
  impl->shrink(new_sz);
}

inline void
Row::swap(Row& y) {
  std::swap(impl, y.impl);



}

inline void
Row::assign(Row& y) {
  impl = y.impl;



}


inline Row&
Row::operator=(const Row& y) {

  Row tmp(y);

  swap(tmp);

  return *this;
}






inline bool
Row::is_line_or_equality() const {
  return type().is_line_or_equality();
}





inline bool
Row::is_ray_or_point_or_inequality() const {
  return type().is_ray_or_point_or_inequality();
}




inline Topology
Row::topology() const {
  return type().topology();
}




inline void
Row::set_is_line_or_equality() {
  impl->type.set_is_line_or_equality();
}





inline void
Row::set_is_ray_or_point_or_inequality() {
  impl->type.set_is_ray_or_point_or_inequality();
}

inline void
Row::set_necessarily_closed() {
  impl->type.set_necessarily_closed();
}

inline void
Row::set_not_necessarily_closed() {
  impl->type.set_not_necessarily_closed();
}





inline Integer&
Row::operator[](size_t k) {
  return (*impl)[k];
}




inline const Integer&
Row::operator[](size_t k) const {
  return (*impl)[k];
}

inline const Integer&
Row::inhomogeneous_term() const {
  return (*this)[0];
}

inline const Integer&
Row::coefficient(size_t k) const {
  return (*this)[k+1];
}


inline bool
operator==(const Row& x, const Row& y) {
  return compare(x, y) == 0;
}


inline bool
operator!=(const Row& x, const Row& y) {
  return compare(x, y) != 0;
}


inline bool
operator<=(const Row& x, const Row& y) {
  return compare(x, y) <= 0;
}


inline bool
operator<(const Row& x, const Row& y) {
  return compare(x, y) < 0;
}


inline bool
operator>=(const Row& x, const Row& y) {
  return compare(x, y) >= 0;
}


inline bool
operator>(const Row& x, const Row& y) {
  return compare(x, y) > 0;
}

}


namespace std {


inline void
swap(Parma_Polyhedra_Library::Row& x, Parma_Polyhedra_Library::Row& y) {
  x.swap(y);
}
inline void
iter_swap(std::vector<Parma_Polyhedra_Library::Row>::iterator x,
          std::vector<Parma_Polyhedra_Library::Row>::iterator y) {
  swap(*x, *y);
}

}
class Parma_Polyhedra_Library::Matrix {
protected:




  Matrix(Topology topol);
  Matrix(Topology topol, size_t n_rows, size_t n_columns);


  Matrix(const Matrix& y);


  virtual ~Matrix();


  Matrix& operator=(const Matrix& y);






  class const_iterator {
  private:
    typedef std::vector<Row>::const_iterator Iter;

    Iter i;

  public:
    typedef std::forward_iterator_tag iterator_category;
    typedef std::iterator_traits<Iter>::value_type value_type;
    typedef std::iterator_traits<Iter>::difference_type difference_type;
    typedef std::iterator_traits<Iter>::pointer pointer;
    typedef std::iterator_traits<Iter>::reference reference;


    const_iterator()
      : i(Iter()) {
    }




    explicit const_iterator(const Iter& b)
      : i(b) {
    }


    const_iterator(const const_iterator& y)
      : i(y.i) {
    }


    const_iterator& operator=(const const_iterator& y) {
      i = y.i;
      return *this;
    }


    reference operator*() const {
      return *i;
    }


    pointer operator->() const {
      return &*i;
    }


    const_iterator& operator++() {
      ++i;
      return *this;
    }


    const_iterator operator++(int) {
      return const_iterator(i++);
    }




    bool operator==(const const_iterator& y) const {
      return i == y.i;
    }




    bool operator!=(const const_iterator& y) const {
      return !operator==(y);
    }
  };

public:




  const_iterator begin() const {
    return const_iterator(rows.begin());
  }


  const_iterator end() const {
    return const_iterator(rows.end());
  }

private:

  std::vector<Row> rows;


  Topology row_topology;


  size_t row_size;




  size_t row_capacity;





  bool sorted;

public:

  void swap(Matrix& y);


  void set_sorted(bool value);


  void set_necessarily_closed();


  void set_not_necessarily_closed();


  void set_rows_topology();
  void grow(size_t new_n_rows, size_t new_n_columns);
  void resize_no_copy(size_t new_n_rows, size_t new_n_columns);






  void add_zero_columns(size_t n);
  void add_rows_and_columns(size_t n);


  void swap_columns(size_t i, size_t j);




  Topology topology() const;




  bool is_necessarily_closed() const;


  bool is_sorted() const;
  size_t space_dimension() const;




  size_t num_columns() const;


  size_t num_rows() const;




  size_t num_lines_or_equalities() const;





  bool check_sorted() const;




  Row& operator[](size_t k);


  const Row& operator[](size_t k) const;



  void normalize();


  void strong_normalize();


  void sort_rows();
  void merge_rows_assign(const Matrix& y);


  void add_row(Row::Type type);


  void add_row(const Row& row);




  void insert(const Row& row);


  void clear();
  virtual void get(std::istream& s);
  virtual void print(std::ostream& s) const;





  void erase_to_end(size_t first_to_erase);
  void sort_and_remove_with_sat(SatMatrix& sat);
  size_t gauss();
  void back_substitute(size_t rank);


  bool OK() const;
};

namespace std {





void swap(Parma_Polyhedra_Library::Matrix& x,
          Parma_Polyhedra_Library::Matrix& y);

}

namespace Parma_Polyhedra_Library {





  bool operator==(const Matrix& x, const Matrix& y);





  bool operator!=(const Matrix& x, const Matrix& y);





  std::istream& operator>>(std::istream& s, Matrix& m);





  std::ostream& operator<<(std::ostream& s, const Matrix& m);

}
namespace Parma_Polyhedra_Library {

inline void
Matrix::swap(Matrix& y) {
  std::swap(rows, y.rows);
  std::swap(row_topology, y.row_topology);
  std::swap(row_size, y.row_size);
  std::swap(row_capacity, y.row_capacity);
  std::swap(sorted, y.sorted);
}

inline
Matrix::Matrix(Topology topol)
  : rows(),
    row_topology(topol),
    row_size(0),
    row_capacity(0),
    sorted(true) {
}


inline
Matrix::~Matrix() {
}


inline Row&
Matrix::operator[](size_t k) {
  assert(k < rows.size());
  return rows[k];
}


inline const Row&
Matrix::operator[](size_t k) const {
  assert(k < rows.size());
  return rows[k];
}


inline size_t
Matrix::num_rows() const {
  return rows.size();
}


inline void
Matrix::set_necessarily_closed() {
  row_topology = NECESSARILY_CLOSED;
  if (num_rows() > 0)
    set_rows_topology();
}

inline void
Matrix::set_not_necessarily_closed() {
  row_topology = NOT_NECESSARILY_CLOSED;
  if (num_rows() > 0)
    set_rows_topology();
}

inline bool
Matrix::is_necessarily_closed() const {
  return row_topology == NECESSARILY_CLOSED;
}

inline Topology
Matrix::topology() const {
  return row_topology;
}

inline void
Matrix::set_sorted(bool value) {
  sorted = value;
}

inline bool
Matrix::is_sorted() const {



  assert(!sorted || check_sorted());
  return sorted;
}


inline size_t
Matrix::num_columns() const {
  return row_size;
}

inline size_t
Matrix::space_dimension() const {
  size_t n_columns = num_columns();
  return (n_columns == 0)
    ? 0
    : n_columns - (is_necessarily_closed() ? 1 : 2);
}



inline bool
operator!=(const Matrix& x, const Matrix& y) {
  return !(x == y);
}


inline void
Matrix::add_zero_columns(size_t n) {
  assert(n > 0);
  grow(num_rows(), num_columns() + n);
}

inline void
Matrix::erase_to_end(size_t first_to_erase) {
  assert(first_to_erase <= rows.size());
  if (first_to_erase < rows.size())
    rows.erase(rows.begin() + first_to_erase, rows.end());
}

inline void
Matrix::clear() {


  std::vector<Row>().swap(rows);
  row_size = 0;
  row_capacity = 0;
  sorted = true;
}

}

namespace std {


inline void
swap(Parma_Polyhedra_Library::Matrix& x,
     Parma_Polyhedra_Library::Matrix& y) {
  x.swap(y);
}

}
class Parma_Polyhedra_Library::Variable {

public:

  explicit Variable(unsigned int i);

  unsigned int id() const;

private:

  unsigned int varid;
};

namespace Parma_Polyhedra_Library {



  std::ostream&
  operator<<(std::ostream& s, const Variable& v);



  bool operator<(const Variable& v, const Variable& w);
}
namespace Parma_Polyhedra_Library {

inline
Variable::Variable(unsigned int i)
  : varid(i) {
}

inline unsigned int
Variable::id() const {
  return varid;
}


inline bool
operator<(const Variable& v, const Variable& w) {
  return v.id() < w.id();
}

}
namespace Parma_Polyhedra_Library {

  bool operator<=(const Polyhedron& x, const Polyhedron& y);
  std::ostream& operator<<(std::ostream& s, const ConSys& cs);
  std::istream& operator>>(std::istream& s, ConSys& cs);
}

namespace std {



void swap(Parma_Polyhedra_Library::ConSys& x,
          Parma_Polyhedra_Library::ConSys& y);

}
class Parma_Polyhedra_Library::ConSys : private Matrix {
public:

  ConSys();


  ConSys(const Constraint& c);


  ConSys(const ConSys& cs);


  virtual ~ConSys();


  ConSys& operator=(const ConSys& y);


  size_t space_dimension() const;




  void clear();




  void insert(const Constraint& c);




  static const ConSys& zero_dim_empty();
  class const_iterator
    : public std::iterator<std::forward_iterator_tag,
                           Constraint,
                           void,
                           const Constraint*,
                           const Constraint&> {
  public:

    const_iterator();


    const_iterator(const const_iterator& y);


    virtual ~const_iterator();


    const_iterator& operator=(const const_iterator& y);


    const Constraint& operator*() const;


    const Constraint* operator->() const;


    const_iterator& operator++();


    const_iterator operator++(int);




    bool operator==(const const_iterator& y) const;




    bool operator!=(const const_iterator& y) const;

  private:
    friend class ConSys;


    Matrix::const_iterator i;


    const Matrix* csp;


    const_iterator(const Matrix::const_iterator& iter, const ConSys& csys);


    void skip_forward();
  };





  const_iterator begin() const;


  const_iterator end() const;


  bool OK() const;

private:
  friend class Parma_Polyhedra_Library::Polyhedron;
  friend bool
  Parma_Polyhedra_Library::operator<=(const Polyhedron& x,
                                      const Polyhedron& y);
  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s, const ConSys& m);
  friend std::istream&
  Parma_Polyhedra_Library::operator>>(std::istream& s, ConSys& cs);
  friend void std::swap(Parma_Polyhedra_Library::ConSys& x,
                        Parma_Polyhedra_Library::ConSys& y);


  ConSys(Topology topol);





  ConSys(Topology topol, size_t n_rows, size_t n_columns);


  void swap(ConSys& y);
  bool adjust_topology_and_dimension(Topology topol,
                                     size_t num_dimensions);




  bool has_strict_inequalities() const;


  Constraint& operator[](size_t k);


  const Constraint& operator[](size_t k) const;


  bool satisfies_all_constraints(const Generator& g) const;


  void affine_preimage(size_t v,
                       const LinExpression& expr,
                       const Integer& denominator);


  size_t num_equalities() const;


  size_t num_inequalities() const;


  void get(std::istream& s);


  void print(std::ostream& s) const;
};
namespace Parma_Polyhedra_Library {

  bool operator<=(const Polyhedron& x, const Polyhedron& y);
  std::ostream& operator<<(std::ostream& s, const GenSys& gs);
  std::istream& operator>>(std::istream& s, GenSys& gs);
}


namespace std {



void swap(Parma_Polyhedra_Library::GenSys& x,
          Parma_Polyhedra_Library::GenSys& y);

}
class Parma_Polyhedra_Library::GenSys : private Matrix {
public:

  GenSys();


  GenSys(const Generator& g);


  GenSys(const GenSys& gs);


  virtual ~GenSys();


  GenSys& operator=(const GenSys& y);


  size_t space_dimension() const;




  void clear();




  void insert(const Generator& g);




  static const GenSys& zero_dim_univ();
  class const_iterator
    : public std::iterator<std::forward_iterator_tag,
                                Generator,
                                void,
                                const Generator*,
                                const Generator&> {
  public:

    const_iterator();


    const_iterator(const const_iterator& y);


    virtual ~const_iterator();


    const_iterator& operator=(const const_iterator& y);


    const Generator& operator*() const;


    const Generator* operator->() const;


    const_iterator& operator++();


    const_iterator operator++(int);




    bool operator==(const const_iterator& y) const;




    bool operator!=(const const_iterator& y) const;

  private:
    friend class GenSys;


    Matrix::const_iterator i;


    const Matrix* gsp;


    const_iterator(const Matrix::const_iterator& iter, const GenSys& gsys);




    void skip_forward();
  };





  const_iterator begin() const;


  const_iterator end() const;


  bool OK() const;

private:
  friend class Parma_Polyhedra_Library::Polyhedron;
  friend bool
  Parma_Polyhedra_Library::operator<=(const Polyhedron& x,
                                      const Polyhedron& y);
  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s, const GenSys& gs);
  friend std::istream&
  Parma_Polyhedra_Library::operator>>(std::istream& s, GenSys& gs);
  friend void std::swap(Parma_Polyhedra_Library::GenSys& x,
                        Parma_Polyhedra_Library::GenSys& y);


  GenSys(Topology topol);





  GenSys(Topology topol, size_t n_rows, size_t n_columns);


  void swap(GenSys& y);
  bool adjust_topology_and_dimension(Topology topol,
                                     size_t num_dimensions);
  void add_corresponding_points();




  bool has_points() const;
  void add_corresponding_closure_points();




  bool has_closure_points() const;


  Generator& operator[](size_t k);


  const Generator& operator[](size_t k) const;




  Parma_Polyhedra_Library::Poly_Con_Relation
  relation_with(const Constraint& c) const;


  void affine_image(size_t v,
                    const LinExpression& expr,
                    const Integer& denominator);


  size_t num_lines() const;


  size_t num_rays() const;







  void remove_invalid_lines_and_rays();


  void get(std::istream& s);


  void print(std::ostream& s) const;
};
namespace Parma_Polyhedra_Library {

  LinExpression operator+(const LinExpression& e1, const LinExpression& e2);
  LinExpression operator+(const Integer& n, const LinExpression& e);
  LinExpression operator+(const LinExpression& e, const Integer& n);

  LinExpression operator-(const LinExpression& e);

  LinExpression operator-(const LinExpression& e1, const LinExpression& e2);
  LinExpression operator-(const Integer& n, const LinExpression& e);
  LinExpression operator-(const LinExpression& e, const Integer& n);

  LinExpression operator*(const Integer& n, const LinExpression& e);
  LinExpression operator*(const LinExpression& e, const Integer& n);

  LinExpression& operator+=(LinExpression& e1, const LinExpression& e2);
  LinExpression& operator+=(LinExpression& e, const Variable& v);
  LinExpression& operator+=(LinExpression& e, const Integer& n);

  LinExpression& operator-=(LinExpression& e1, const LinExpression& e2);
  LinExpression& operator-=(LinExpression& e, const Variable& v);
  LinExpression& operator-=(LinExpression& e, const Integer& n);
}


namespace std {



void swap(Parma_Polyhedra_Library::LinExpression& x,
          Parma_Polyhedra_Library::LinExpression& y);

}
class Parma_Polyhedra_Library::LinExpression : private Row {
public:

  LinExpression();


  LinExpression(const LinExpression& e);


  virtual ~LinExpression();




  explicit LinExpression(const Integer& n);




  LinExpression(const Variable& v);
  explicit LinExpression(const Constraint& c);
  explicit LinExpression(const Generator& g);


  size_t space_dimension() const;


  static const LinExpression& zero();

private:
  friend class Parma_Polyhedra_Library::Constraint;
  friend class Parma_Polyhedra_Library::Generator;
  friend class Parma_Polyhedra_Library::Polyhedron;
  friend void
  Parma_Polyhedra_Library::ConSys::affine_preimage(size_t v,
                                                   const LinExpression& expr,
                                                   const Integer& denominator);
  friend void
  Parma_Polyhedra_Library::GenSys::affine_image(size_t v,
                                                const LinExpression& expr,
                                                const Integer& denominator);
  friend void std::swap(Parma_Polyhedra_Library::LinExpression& x,
                       Parma_Polyhedra_Library::LinExpression& y);


  LinExpression(const LinExpression& e, size_t sz);






  LinExpression(size_t sz, bool);


  void swap(LinExpression& y);


  friend LinExpression
  Parma_Polyhedra_Library::operator+(const LinExpression& e1,
                                     const LinExpression& e2);


  friend LinExpression
  Parma_Polyhedra_Library::operator+(const Integer& n,
                                     const LinExpression& e);


  friend LinExpression
  Parma_Polyhedra_Library::operator+(const LinExpression& e,
                                     const Integer& n);


  friend LinExpression
  Parma_Polyhedra_Library::operator-(const LinExpression& e);


  friend LinExpression
  Parma_Polyhedra_Library::operator-(const LinExpression& e1,
                                     const LinExpression& e2);


  friend LinExpression
  Parma_Polyhedra_Library::operator-(const Integer& n,
                                     const LinExpression& e);


  friend LinExpression
  Parma_Polyhedra_Library::operator-(const LinExpression& e,
                                     const Integer& n);


  friend LinExpression
  Parma_Polyhedra_Library::operator*(const Integer& n,
                                     const LinExpression& e);


  friend LinExpression
  Parma_Polyhedra_Library::operator*(const LinExpression& e,
                                     const Integer& n);


  friend LinExpression&
  Parma_Polyhedra_Library::operator+=(LinExpression& e1,
                                      const LinExpression& e2);


  friend LinExpression&
  Parma_Polyhedra_Library::operator+=(LinExpression& e,
                                      const Variable& v);


  friend LinExpression&
  Parma_Polyhedra_Library::operator+=(LinExpression& e,
                                      const Integer& n);


  friend LinExpression&
  Parma_Polyhedra_Library::operator-=(LinExpression& e1,
                                      const LinExpression& e2);


  friend LinExpression&
  Parma_Polyhedra_Library::operator-=(LinExpression& e,
                                      const Variable& v);


  friend LinExpression&
  Parma_Polyhedra_Library::operator-=(LinExpression& e,
                                      const Integer& n);
};
namespace Parma_Polyhedra_Library {

inline
LinExpression::LinExpression()
  : Row(Row::Type(), 1) {
}

inline
LinExpression::LinExpression(size_t sz, bool)
  : Row(Row::Type(), sz) {
}

inline
LinExpression::LinExpression(const Variable& v)
  : Row(Row::Type(), v.id() + 2) {
  (*this)[v.id() + 1] = 1;
}

inline
LinExpression::LinExpression(const LinExpression& e)
  : Row(e) {
}

inline
LinExpression::~LinExpression() {
}

inline
LinExpression::LinExpression(const LinExpression& e, size_t sz)
  : Row(e, sz, sz) {
}

inline
LinExpression::LinExpression(const Integer& n)
  : Row(Row::Type(), 1) {
  (*this)[0] = n;
}

inline size_t
LinExpression::space_dimension() const {
  return size() - 1;
}

inline const LinExpression&
LinExpression::zero() {
  static LinExpression z = LinExpression(Integer_zero());
  return z;
}

inline LinExpression
operator+(const LinExpression& e, const Integer& n) {
  return n + e;
}

inline LinExpression
operator-(const LinExpression& e, const Integer& n) {
  return -n + e;
}

inline LinExpression
operator*(const LinExpression& e, const Integer& n) {
  return n * e;
}

inline LinExpression&
operator+=(LinExpression& e, const Integer& n) {
  e[0] += n;
  return e;
}

inline LinExpression&
operator-=(LinExpression& e, const Integer& n) {
  e[0] -= n;
  return e;
}

inline void
LinExpression::swap(LinExpression& y) {
  Row::swap(y);
}

}


namespace std {


inline void
swap(Parma_Polyhedra_Library::LinExpression& x,
     Parma_Polyhedra_Library::LinExpression& y) {
  x.swap(y);
}

}
namespace Parma_Polyhedra_Library {


  std::ostream& operator<<(std::ostream& s, const Constraint& c);


  Constraint operator==(const LinExpression& e1, const LinExpression& e2);
  Constraint operator==(const LinExpression& e, const Integer& n);
  Constraint operator==(const Integer& n, const LinExpression& e);

  Constraint operator<=(const LinExpression& e1, const LinExpression& e2);
  Constraint operator<=(const LinExpression& e, const Integer& n);
  Constraint operator<=(const Integer& n, const LinExpression& e);

  Constraint operator>=(const LinExpression& e1, const LinExpression& e2);
  Constraint operator>=(const LinExpression& e, const Integer& n);
  Constraint operator>=(const Integer& n, const LinExpression& e);

  Constraint operator<(const LinExpression& e1, const LinExpression& e2);
  Constraint operator<(const LinExpression& e, const Integer& n);
  Constraint operator<(const Integer& n, const LinExpression& e);

  Constraint operator>(const LinExpression& e1, const LinExpression& e2);
  Constraint operator>(const LinExpression& e, const Integer& n);
  Constraint operator>(const Integer& n, const LinExpression& e);

  Constraint operator>>(const Constraint& c, unsigned int offset);
}

namespace std {



void swap(Parma_Polyhedra_Library::Constraint& x,
          Parma_Polyhedra_Library::Constraint& y);

}
class Parma_Polyhedra_Library::Constraint : private Row {
public:

  Constraint(const Constraint& c);


  ~Constraint();


  Constraint& operator=(const Constraint& c);


  size_t space_dimension() const;


  enum Type {

    EQUALITY,

    NONSTRICT_INEQUALITY,

    STRICT_INEQUALITY
  };


  Type type() const;




  bool is_equality() const;




  bool is_inequality() const;




  bool is_nonstrict_inequality() const;




  bool is_strict_inequality() const;






  const Integer& coefficient(Variable v) const;


  const Integer& inhomogeneous_term() const;


  static const Constraint& zero_dim_false();




  static const Constraint& zero_dim_positivity();


  bool OK() const;

private:
  friend class Parma_Polyhedra_Library::ConSys;
  friend class Parma_Polyhedra_Library::ConSys::const_iterator;
  friend class Parma_Polyhedra_Library::Polyhedron;
  friend const Integer&
  Parma_Polyhedra_Library::operator*(const Constraint& c, const Generator& g);
  friend const Integer&
  Parma_Polyhedra_Library::reduced_scalar_product(const Constraint& c,
                                                  const Generator& g);
  friend
  Parma_Polyhedra_Library::LinExpression::LinExpression(const Constraint& c);
  friend void std::swap(Parma_Polyhedra_Library::Constraint& x,
                       Parma_Polyhedra_Library::Constraint& y);


  Constraint();




  Constraint(LinExpression& e);




  Constraint(Row::Type t, size_t sz);


  void swap(Constraint& y);




  void
  throw_dimension_incompatible(const char* method, Variable v) const;


  friend Constraint
  Parma_Polyhedra_Library::operator==(const LinExpression& e1,
                                      const LinExpression& e2);


  friend Constraint
  Parma_Polyhedra_Library::operator==(const LinExpression& e,
                                      const Integer& n);


  friend Constraint
  Parma_Polyhedra_Library::operator==(const Integer& n,
                                      const LinExpression& e);


  friend Constraint
  Parma_Polyhedra_Library::operator>=(const LinExpression& e1,
                                      const LinExpression& e2);


  friend Constraint
  Parma_Polyhedra_Library::operator>=(const LinExpression& e,
                                      const Integer& n);


  friend Constraint
  Parma_Polyhedra_Library::operator>=(const Integer& n,
                                      const LinExpression& e);


  friend Constraint
  Parma_Polyhedra_Library::operator<=(const LinExpression& e1,
                                      const LinExpression& e2);


  friend Constraint
  Parma_Polyhedra_Library::operator<=(const LinExpression& e,
                                      const Integer& n);


  friend Constraint
  Parma_Polyhedra_Library::operator<=(const Integer& n,
                                      const LinExpression& e);


  friend Constraint
  Parma_Polyhedra_Library::operator>(const LinExpression& e1,
                                     const LinExpression& e2);


  friend Constraint
  Parma_Polyhedra_Library::operator>(const LinExpression& e,
                                     const Integer& n);


  friend Constraint
  Parma_Polyhedra_Library::operator>(const Integer& n,
                                     const LinExpression& e);


  friend Constraint
  Parma_Polyhedra_Library::operator<(const LinExpression& e1,
                                     const LinExpression& e2);


  friend Constraint
  Parma_Polyhedra_Library::operator<(const LinExpression& e,
                                     const Integer& n);


  friend Constraint
  Parma_Polyhedra_Library::operator<(const Integer& n,
                                     const LinExpression& e);




  friend Constraint
  Parma_Polyhedra_Library::operator>>(const Constraint& c,
                                      unsigned int offset);


  Constraint(const Constraint& c, size_t sz);




  static Constraint construct_epsilon_geq_zero();


  static const Constraint& epsilon_geq_zero();




  static const Constraint& epsilon_leq_one();
  bool is_trivial_true() const;
  bool is_trivial_false() const;


  void set_is_equality();


  void set_is_inequality();
};
namespace Parma_Polyhedra_Library {

inline
Constraint::Constraint(LinExpression& e) {
  Row::swap(e);
}

inline
Constraint::Constraint(const Constraint& c)
  : Row(c) {
}

inline
Constraint::Constraint(const Constraint& c, size_t sz)
  : Row(c, sz, sz) {
}

inline
Constraint::Constraint(Row::Type t, size_t sz)
  : Row(t, sz) {
}

inline
Constraint::~Constraint() {
}

inline Constraint&
Constraint::operator=(const Constraint& c) {
  Row::operator=(c);
  return *this;
}

inline size_t
Constraint::space_dimension() const {
  return Row::space_dimension();
}

inline bool
Constraint::is_equality() const {
  return is_line_or_equality();
}

inline bool
Constraint::is_inequality() const {
  return is_ray_or_point_or_inequality();
}

inline Constraint::Type
Constraint::type() const {
  if (is_equality())
    return EQUALITY;
  if (is_necessarily_closed())
    return NONSTRICT_INEQUALITY;
  else
    return ((*this)[size() - 1] < 0)
      ? STRICT_INEQUALITY
      : NONSTRICT_INEQUALITY;
}

inline bool
Constraint::is_nonstrict_inequality() const {
  return type() == NONSTRICT_INEQUALITY;
}

inline bool
Constraint::is_strict_inequality() const {
  return type() == STRICT_INEQUALITY;
}

inline void
Constraint::set_is_equality() {
  set_is_line_or_equality();
}

inline void
Constraint::set_is_inequality() {
  set_is_ray_or_point_or_inequality();
}

inline const Integer&
Constraint::coefficient(Variable v) const {
  if (v.id() >= space_dimension())
    throw_dimension_incompatible("PPL::Constraint::coefficient(v)", v);
  return Row::coefficient(v.id());
}

inline const Integer&
Constraint::inhomogeneous_term() const {
  return Row::inhomogeneous_term();
}

inline Constraint
operator==(const LinExpression& e1, const LinExpression& e2) {
  LinExpression diff = e1 - e2;
  Constraint c(diff);
  c.set_is_equality();




  c.normalize();

  return c;
}

inline Constraint
operator>=(const LinExpression& e1, const LinExpression& e2) {
  LinExpression diff = e1 - e2;
  Constraint c(diff);
  c.set_is_inequality();

  c.normalize();
  return c;
}

inline Constraint
operator>(const LinExpression& e1, const LinExpression& e2) {
  LinExpression diff;


  size_t e1_dim = e1.space_dimension();
  size_t e2_dim = e2.space_dimension();
  if (e1_dim > e2_dim)
    diff -= Variable(e1_dim);
  else
    diff -= Variable(e2_dim);
  diff += e1;
  diff -= e2;

  Constraint c(diff);
  c.set_not_necessarily_closed();
  c.set_is_inequality();
  return c;
}

inline Constraint
operator==(const Integer& n, const LinExpression& e) {
  LinExpression diff = n - e;
  Constraint c(diff);
  c.set_is_equality();




  c.normalize();

  return c;
}

inline Constraint
operator>=(const Integer& n, const LinExpression& e) {
  LinExpression diff = n - e;
  Constraint c(diff);
  c.set_is_inequality();

  c.normalize();
  return c;
}

inline Constraint
operator>(const Integer& n, const LinExpression& e) {
  LinExpression diff;


  diff -= Variable(e.space_dimension());
  diff += n;
  diff -= e;

  Constraint c(diff);
  c.set_not_necessarily_closed();
  c.set_is_inequality();
  return c;
}

inline Constraint
operator==(const LinExpression& e, const Integer& n) {
  LinExpression diff = e - n;
  Constraint c(diff);
  c.set_is_equality();




  c.normalize();

  return c;
}

inline Constraint
operator>=(const LinExpression& e, const Integer& n) {
  LinExpression diff = e - n;
  Constraint c(diff);
  c.set_is_inequality();

  c.normalize();
  return c;
}

inline Constraint
operator>(const LinExpression& e, const Integer& n) {
  LinExpression diff;


  diff -= Variable(e.space_dimension());
  diff += e;
  diff -= n;

  Constraint c(diff);
  c.set_not_necessarily_closed();
  c.set_is_inequality();
  return c;
}

inline Constraint
operator<=(const LinExpression& e1, const LinExpression& e2) {
  return e2 >= e1;
}

inline Constraint
operator<=(const Integer& n, const LinExpression& e) {
  return e >= n;
}

inline Constraint
operator<=(const LinExpression& e, const Integer& n) {
  return n >= e;
}

inline Constraint
operator<(const LinExpression& e1, const LinExpression& e2) {
  return e2 > e1;
}

inline Constraint
operator<(const Integer& n, const LinExpression& e) {
  return e > n;
}

inline Constraint
operator<(const LinExpression& e, const Integer& n) {
  return n > e;
}

inline const Constraint&
Constraint::zero_dim_false() {
  static Constraint zdf(LinExpression::zero() == Integer_one());
  return zdf;
}

inline const Constraint&
Constraint::zero_dim_positivity() {
  static Constraint zdp(LinExpression::zero() <= Integer_one());
  return zdp;
}

inline const Constraint&
Constraint::epsilon_geq_zero() {
  static Constraint eps_geq_zero = construct_epsilon_geq_zero();
  return eps_geq_zero;
}

inline const Constraint&
Constraint::epsilon_leq_one() {
  static Constraint eps_leq_one(LinExpression::zero() < Integer_one());
  return eps_leq_one;
}

inline void
Constraint::swap(Constraint& y) {
  Row::swap(y);
}

}

namespace std {




inline void
swap(Parma_Polyhedra_Library::Constraint& x,
     Parma_Polyhedra_Library::Constraint& y) {
  x.swap(y);
}

}
namespace Parma_Polyhedra_Library {

inline
ConSys::ConSys()
  : Matrix(NECESSARILY_CLOSED) {
}

inline
ConSys::ConSys(const Constraint& c)
  : Matrix(c.topology()) {
  Matrix::insert(c);
}

inline
ConSys::ConSys(const ConSys& cs)
  : Matrix(cs) {
}

inline
ConSys::ConSys(Topology topol)
  : Matrix(topol) {
}

inline
ConSys::ConSys(Topology topol, size_t n_rows, size_t n_columns)
  : Matrix(topol, n_rows, n_columns) {
}

inline
ConSys::~ConSys() {
}

inline ConSys&
ConSys::operator=(const ConSys& y) {
  Matrix::operator=(y);
  return *this;
}

inline Constraint&
ConSys::operator[](size_t k) {
  return static_cast<Constraint&> (Matrix::operator[](k));
}

inline const Constraint&
ConSys::operator[](size_t k) const {
  return static_cast<const Constraint&> (Matrix::operator[](k));
}

inline size_t
ConSys::space_dimension() const {
  return Matrix::space_dimension();
}

inline void
ConSys::clear() {
  Matrix::clear();
}

inline const ConSys&
ConSys::zero_dim_empty() {
  static ConSys zdf(Constraint::zero_dim_false());
  return zdf;
}

inline
ConSys::const_iterator::const_iterator(const Matrix::const_iterator& iter,
                                       const ConSys& csys)
  : i(iter), csp(&csys) {
}

inline
ConSys::const_iterator::const_iterator()
  : i(), csp(0) {
}

inline
ConSys::const_iterator::const_iterator(const const_iterator& y)
  : i(y.i), csp(y.csp) {
}

inline
ConSys::const_iterator::~const_iterator() {
}

inline ConSys::const_iterator&
ConSys::const_iterator::operator=(const const_iterator& y) {
  i = y.i;
  csp = y.csp;
  return *this;
}

inline const Constraint&
ConSys::const_iterator::operator*() const {
  return static_cast<const Constraint&>(*i);
}

inline const Constraint*
ConSys::const_iterator::operator->() const {
  return static_cast<const Constraint*>(i.operator->());
}

inline ConSys::const_iterator&
ConSys::const_iterator::operator++() {
  ++i;
  skip_forward();
  return *this;
}

inline ConSys::const_iterator
ConSys::const_iterator::operator++(int) {
  const_iterator tmp = *this;
  operator++();
  return tmp;
}

inline bool
ConSys::const_iterator::operator==(const const_iterator& y) const {
  return i == y.i;
}

inline bool
ConSys::const_iterator::operator!=(const const_iterator& y) const {
  return i != y.i;
}

inline ConSys::const_iterator
ConSys::begin() const {
  const_iterator i(Matrix::begin(), *this);
  i.skip_forward();
  return i;
}

inline ConSys::const_iterator
ConSys::end() const {
  const_iterator i(Matrix::end(), *this);
  return i;
}

inline void
ConSys::swap(ConSys& y) {
  Matrix::swap(y);
}

}


namespace std {


inline void
swap(Parma_Polyhedra_Library::ConSys& x,
     Parma_Polyhedra_Library::ConSys& y) {
  x.swap(y);
}

}
namespace Parma_Polyhedra_Library {


bool operator<=(const Polyhedron& x, const Polyhedron& y);
std::ostream& operator<<(std::ostream& s, const Generator& g);

}

namespace std {



void swap(Parma_Polyhedra_Library::Generator& x,
          Parma_Polyhedra_Library::Generator& y);

}
class Parma_Polyhedra_Library::Generator : private Row {
public:






  static Generator line(const LinExpression& e);







  static Generator ray(const LinExpression& e);







  static Generator point(const LinExpression& e = LinExpression::zero(),
                         const Integer& d = Integer_one());







  static Generator
  closure_point(const LinExpression& e = LinExpression::zero(),
                const Integer& d = Integer_one());


  Generator(const Generator& g);


  ~Generator();


  Generator& operator=(const Generator& g);


  size_t space_dimension() const;


  enum Type {

    LINE,

    RAY,

    POINT,

    CLOSURE_POINT
  };


  Type type() const;


  bool is_line() const;


  bool is_ray() const;


  bool is_point() const;


  bool is_closure_point() const;






  const Integer& coefficient(Variable v) const;






  const Integer& divisor() const;


  static const Generator& zero_dim_point();




  static const Generator& zero_dim_closure_point();


  bool OK() const;

private:



  Generator(LinExpression& e);


  void swap(Generator& y);




  void
  throw_dimension_incompatible(const char* method, Variable v) const;




  void
  throw_invalid_argument(const char* method, const char* reason) const;

  friend class Parma_Polyhedra_Library::GenSys;
  friend class Parma_Polyhedra_Library::GenSys::const_iterator;
  friend class Parma_Polyhedra_Library::Polyhedron;
  friend const Integer&
  Parma_Polyhedra_Library::operator*(const Constraint& c, const Generator& g);
  friend const Integer&
  Parma_Polyhedra_Library::reduced_scalar_product(const Constraint& c,
                                                  const Generator& g);
  friend
  Parma_Polyhedra_Library::LinExpression::LinExpression(const Generator& g);
  friend
  bool Parma_Polyhedra_Library
  ::ConSys::satisfies_all_constraints(const Generator& g)
 const;
  friend bool Parma_Polyhedra_Library::operator<=(const Polyhedron& x,
                                                  const Polyhedron& y);
  friend void std::swap(Parma_Polyhedra_Library::Generator& x,
                        Parma_Polyhedra_Library::Generator& y);


  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s, const Generator& g);


  Generator(const Generator& g, size_t sz);


  bool is_ray_or_point() const;


  void set_is_line();


  void set_is_ray_or_point();




  bool is_matching_closure_point(const Generator& p) const;


  Generator();
};

namespace Parma_Polyhedra_Library {


  Generator line(const LinExpression& e);


  Generator ray(const LinExpression& e);


  Generator point(const LinExpression& e = LinExpression::zero(),
                  const Integer& d = Integer_one());




  Generator closure_point(const LinExpression& e = LinExpression::zero(),
                          const Integer& d = Integer_one());

}
namespace Parma_Polyhedra_Library {

inline
Generator::Generator(LinExpression& e) {
  Row::swap(e);
}

inline
Generator::Generator(const Generator& g)
  : Row(g) {
}

inline
Generator::Generator(const Generator& g, size_t sz)
  : Row(g, sz, sz) {
}

inline
Generator::~Generator() {
}

inline Generator&
Generator::operator=(const Generator& g) {
  Row::operator=(g);
  return *this;
}

inline size_t
Generator::space_dimension() const {
  return Row::space_dimension();
}

inline bool
Generator::is_line() const {
  return is_line_or_equality();
}

inline bool
Generator::is_ray_or_point() const {
  return is_ray_or_point_or_inequality();
}

inline bool
Generator::is_ray() const {
  return is_ray_or_point() && ((*this)[0] == 0);
}

inline Generator::Type
Generator::type() const {
  if (is_line())
    return LINE;
  const Generator& g = *this;
  if (g[0] == 0)
    return RAY;
  if (is_necessarily_closed())
    return POINT;
  else

    return (g[size() - 1] == 0) ? CLOSURE_POINT : POINT;
}

inline bool
Generator::is_point() const {
  return type() == POINT;
}

inline bool
Generator::is_closure_point() const {
  return type() == CLOSURE_POINT;
}

inline void
Generator::set_is_line() {
  set_is_line_or_equality();
}

inline void
Generator::set_is_ray_or_point() {
  set_is_ray_or_point_or_inequality();
}

inline const Integer&
Generator::coefficient(Variable v) const {
  if (v.id() >= space_dimension())
    throw_dimension_incompatible("PPL::Generator::coefficient(v)", v);
  return Row::coefficient(v.id());
}

inline const Integer&
Generator::divisor() const {
  const Integer& d = Row::inhomogeneous_term();
  if (!is_ray_or_point() || d == 0)
    throw_invalid_argument("PPL::Generator::divisor()",
                           "*this is is neither a point "
                           "nor a closure point");
  return d;
}


inline const Generator&
Generator::zero_dim_point() {
  static Generator zdp = point();
  return zdp;
}

inline const Generator&
Generator::zero_dim_closure_point() {
  static Generator zdcp = closure_point();
  return zdcp;
}

inline Generator
line(const LinExpression& e) {
  return Generator::line(e);
}

inline Generator
ray(const LinExpression& e) {
  return Generator::ray(e);
}

inline Generator
point(const LinExpression& e, const Integer& d) {
  return Generator::point(e, d);
}

inline Generator
closure_point(const LinExpression& e, const Integer& d) {
  return Generator::closure_point(e, d);
}

inline void
Generator::swap(Generator& y) {
  Row::swap(y);
}

}

namespace std {


inline void
swap(Parma_Polyhedra_Library::Generator& x,
     Parma_Polyhedra_Library::Generator& y) {
  x.swap(y);
}

}
namespace Parma_Polyhedra_Library {

inline
GenSys::GenSys()
  : Matrix(NECESSARILY_CLOSED) {
}

inline
GenSys::GenSys(const Generator& g)
  : Matrix(g.topology()) {
  Matrix::insert(g);
}

inline
GenSys::GenSys(const GenSys& gs)
  : Matrix(gs) {
}

inline
GenSys::GenSys(Topology topol)
  : Matrix(topol) {
}

inline
GenSys::GenSys(Topology topol, size_t n_rows, size_t n_columns)
  : Matrix(topol, n_rows, n_columns) {
}

inline
GenSys::~GenSys() {
}

inline GenSys&
GenSys::operator=(const GenSys& y) {
  Matrix::operator=(y);
  return *this;
}

inline size_t
GenSys::space_dimension() const {
  return Matrix::space_dimension();
}

inline void
GenSys::clear() {
  Matrix::clear();
}

inline Generator&
GenSys::operator[](size_t k) {
  return static_cast<Generator&>(Matrix::operator[](k));
}

inline const Generator&
GenSys::operator[](size_t k) const {
  return static_cast<const Generator&>(Matrix::operator[](k));
}

inline
GenSys::const_iterator::const_iterator(const Matrix::const_iterator& iter,
                                       const GenSys& gsys)
  : i(iter), gsp(&gsys) {
}

inline
GenSys::const_iterator::const_iterator()
  : i(), gsp(0) {
}

inline
GenSys::const_iterator::const_iterator(const const_iterator& y)
  : i(y.i), gsp(y.gsp) {
}

inline
GenSys::const_iterator::~const_iterator() {
}

inline
GenSys::const_iterator&
GenSys::const_iterator::operator=(const const_iterator& y) {
  i = y.i;
  gsp = y.gsp;
  return *this;
}

inline const Generator&
GenSys::const_iterator::operator*() const {
  return static_cast<const Generator&>(*i);
}

inline const Generator*
GenSys::const_iterator::operator->() const {
  return static_cast<const Generator*>(i.operator->());
}

inline GenSys::const_iterator&
GenSys::const_iterator::operator++() {
  ++i;
  if (!gsp->is_necessarily_closed())
    skip_forward();
  return *this;
}

inline GenSys::const_iterator
GenSys::const_iterator::operator++(int) {
  const_iterator tmp = *this;
  operator++();
  return tmp;
}

inline bool
GenSys::const_iterator::operator==(const const_iterator& y) const {
  return i == y.i;
}

inline bool
GenSys::const_iterator::operator!=(const const_iterator& y) const {
  return i != y.i;
}

inline GenSys::const_iterator
GenSys::begin() const {
  const_iterator i(Matrix::begin(), *this);
  if (!is_necessarily_closed())
    i.skip_forward();
  return i;
}

inline GenSys::const_iterator
GenSys::end() const {
  const_iterator i(Matrix::end(), *this);
  return i;
}

inline const GenSys&
GenSys::zero_dim_univ() {
  static GenSys zdu(Generator::zero_dim_point());
  return zdu;
}

inline void
GenSys::swap(GenSys& y) {
  Matrix::swap(y);
}

}


namespace std {


inline void
swap(Parma_Polyhedra_Library::GenSys& x,
     Parma_Polyhedra_Library::GenSys& y) {
  x.swap(y);
}

}
namespace Parma_Polyhedra_Library {

  std::ostream& operator<<(std::ostream& s, const Poly_Con_Relation& r);
  bool operator==(const Poly_Con_Relation& x, const Poly_Con_Relation& y);
  bool operator!=(const Poly_Con_Relation& x, const Poly_Con_Relation& y);
  Poly_Con_Relation operator&&(const Poly_Con_Relation& x,
                               const Poly_Con_Relation& y);
  Poly_Con_Relation operator-(const Poly_Con_Relation& x,
                              const Poly_Con_Relation& y);
}






class Parma_Polyhedra_Library::Poly_Con_Relation {
private:

  typedef unsigned int flags_t;



  static const flags_t NOTHING = 0U;
  static const flags_t IS_DISJOINT = 1U << 0;
  static const flags_t STRICTLY_INTERSECTS = 1U << 1;
  static const flags_t IS_INCLUDED = 1U << 2;
  static const flags_t SATURATES = 1U << 3;



  static const flags_t EVERYTHING
  = IS_DISJOINT
  | STRICTLY_INTERSECTS
  | IS_INCLUDED
  | SATURATES;


  flags_t flags;


  static bool implies(flags_t x, flags_t y);


  Poly_Con_Relation(flags_t mask);


  void print(std::ostream& s) const;


  friend bool
  Parma_Polyhedra_Library::operator==(const Poly_Con_Relation& x,
                                      const Poly_Con_Relation& y);


  friend bool
  Parma_Polyhedra_Library::operator!=(const Poly_Con_Relation& x,
                                      const Poly_Con_Relation& y);


  friend Poly_Con_Relation
  Parma_Polyhedra_Library::operator&&(const Poly_Con_Relation& x,
                                      const Poly_Con_Relation& y);


  friend Poly_Con_Relation
  Parma_Polyhedra_Library::operator-(const Poly_Con_Relation& x,
                                     const Poly_Con_Relation& y);


  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s,
                                      const Poly_Con_Relation& r);

public:





  flags_t get_flags() const;

public:

  static Poly_Con_Relation nothing();




  static Poly_Con_Relation is_disjoint();




  static Poly_Con_Relation strictly_intersects();




  static Poly_Con_Relation is_included();




  static Poly_Con_Relation saturates();


  bool implies(const Poly_Con_Relation& y) const;


  bool OK() const;
};
namespace Parma_Polyhedra_Library {

inline
Poly_Con_Relation::Poly_Con_Relation(flags_t mask)
  : flags(mask) {
}

inline Poly_Con_Relation::flags_t
Poly_Con_Relation::get_flags() const {
  return flags;
}

inline Poly_Con_Relation
Poly_Con_Relation::nothing() {
  return Poly_Con_Relation(NOTHING);
}

inline Poly_Con_Relation
Poly_Con_Relation::is_disjoint() {
  return Poly_Con_Relation(IS_DISJOINT);
}

inline Poly_Con_Relation
Poly_Con_Relation::strictly_intersects() {
  return Poly_Con_Relation(STRICTLY_INTERSECTS);
}

inline Poly_Con_Relation
Poly_Con_Relation::is_included() {
  return Poly_Con_Relation(IS_INCLUDED);
}

inline Poly_Con_Relation
Poly_Con_Relation::saturates() {
  return Poly_Con_Relation(SATURATES);
}

inline bool
Poly_Con_Relation::implies(flags_t x, flags_t y) {
  return (x & y) == y;
}

inline bool
Poly_Con_Relation::implies(const Poly_Con_Relation& y) const {
  return implies(flags, y.flags);
}

inline bool
operator==(const Poly_Con_Relation& x, const Poly_Con_Relation& y) {
  return x.flags == y.flags;
}

inline bool
operator!=(const Poly_Con_Relation& x, const Poly_Con_Relation& y) {
  return x.flags != y.flags;
}

inline Poly_Con_Relation
operator&&(const Poly_Con_Relation& x, const Poly_Con_Relation& y) {
  return Poly_Con_Relation(x.flags | y.flags);
}

inline Poly_Con_Relation
operator-(const Poly_Con_Relation& x, const Poly_Con_Relation& y) {
  return Poly_Con_Relation(x.flags & ~y.flags);
}

}
namespace Parma_Polyhedra_Library {

  std::ostream& operator<<(std::ostream& s, const Poly_Gen_Relation& r);
  bool operator==(const Poly_Gen_Relation& x, const Poly_Gen_Relation& y);
  bool operator!=(const Poly_Gen_Relation& x, const Poly_Gen_Relation& y);
  Poly_Gen_Relation operator&&(const Poly_Gen_Relation& x,
                               const Poly_Gen_Relation& y);
  Poly_Gen_Relation operator-(const Poly_Gen_Relation& x,
                              const Poly_Gen_Relation& y);
}






class Parma_Polyhedra_Library::Poly_Gen_Relation {
private:

  typedef unsigned int flags_t;



  static const flags_t NOTHING = 0U;
  static const flags_t SUBSUMES = 1U << 0;



  static const flags_t EVERYTHING
  = SUBSUMES;


  flags_t flags;


  static bool implies(flags_t x, flags_t y);


  Poly_Gen_Relation(flags_t mask);


  void print(std::ostream& s) const;


  friend bool
  Parma_Polyhedra_Library::operator==(const Poly_Gen_Relation& x,
                                      const Poly_Gen_Relation& y);


  friend bool
  Parma_Polyhedra_Library::operator!=(const Poly_Gen_Relation& x,
                                      const Poly_Gen_Relation& y);


  friend Poly_Gen_Relation
  Parma_Polyhedra_Library::operator&&(const Poly_Gen_Relation& x,
                                      const Poly_Gen_Relation& y);


  friend Poly_Gen_Relation
  Parma_Polyhedra_Library::operator-(const Poly_Gen_Relation& x,
                                     const Poly_Gen_Relation& y);


  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s,
                                      const Poly_Gen_Relation& r);

public:





  flags_t get_flags() const;

public:

  static Poly_Gen_Relation nothing();


  static Poly_Gen_Relation subsumes();


  bool implies(const Poly_Gen_Relation& y) const;


  bool OK() const;
};
namespace Parma_Polyhedra_Library {

inline
Poly_Gen_Relation::Poly_Gen_Relation(flags_t mask)
  : flags(mask) {
}

inline Poly_Gen_Relation::flags_t
Poly_Gen_Relation::get_flags() const {
  return flags;
}

inline Poly_Gen_Relation
Poly_Gen_Relation::nothing() {
  return Poly_Gen_Relation(NOTHING);
}

inline Poly_Gen_Relation
Poly_Gen_Relation::subsumes() {
  return Poly_Gen_Relation(SUBSUMES);
}

inline bool
Poly_Gen_Relation::implies(flags_t x, flags_t y) {
  return (x & y) == y;
}

inline bool
Poly_Gen_Relation::implies(const Poly_Gen_Relation& y) const {
  return implies(flags, y.flags);
}

inline bool
operator==(const Poly_Gen_Relation& x, const Poly_Gen_Relation& y) {
  return x.flags == y.flags;
}

inline bool
operator!=(const Poly_Gen_Relation& x, const Poly_Gen_Relation& y) {
  return x.flags != y.flags;
}

inline Poly_Gen_Relation
operator&&(const Poly_Gen_Relation& x, const Poly_Gen_Relation& y) {
  return Poly_Gen_Relation(x.flags | y.flags);
}

inline Poly_Gen_Relation
operator-(const Poly_Gen_Relation& x, const Poly_Gen_Relation& y) {
  return Poly_Gen_Relation(x.flags & ~y.flags);
}

}
namespace Parma_Polyhedra_Library {

  Status operator&(const Status& x, const Status& y);
  Status operator|(const Status& x, const Status& y);
}
class Parma_Polyhedra_Library::Status {
public:

  Status();


  friend Status
  Parma_Polyhedra_Library::operator&(const Status& x, const Status& y);


  friend Status
  Parma_Polyhedra_Library::operator|(const Status& x, const Status& y);



  bool test_zero_dim_univ() const;
  void reset_zero_dim_univ();
  void set_zero_dim_univ();

  bool test_empty() const;
  void reset_empty();
  void set_empty();

  bool test_c_up_to_date() const;
  void reset_c_up_to_date();
  void set_c_up_to_date();

  bool test_g_up_to_date() const;
  void reset_g_up_to_date();
  void set_g_up_to_date();

  bool test_c_minimized() const;
  void reset_c_minimized();
  void set_c_minimized();

  bool test_g_minimized() const;
  void reset_g_minimized();
  void set_g_minimized();

  bool test_sat_c_up_to_date() const;
  void reset_sat_c_up_to_date();
  void set_sat_c_up_to_date();

  bool test_sat_g_up_to_date() const;
  void reset_sat_g_up_to_date();
  void set_sat_g_up_to_date();



  bool OK() const;

private:

  typedef unsigned int flags_t;



  static const flags_t ZERO_DIM_UNIV = 0U;
  static const flags_t EMPTY = 1U << 0;
  static const flags_t C_UP_TO_DATE = 1U << 1;
  static const flags_t G_UP_TO_DATE = 1U << 2;
  static const flags_t C_MINIMIZED = 1U << 3;
  static const flags_t G_MINIMIZED = 1U << 4;
  static const flags_t SAT_C_UP_TO_DATE = 1U << 5;
  static const flags_t SAT_G_UP_TO_DATE = 1U << 6;



  flags_t flags;


  Status(flags_t mask);


  bool test_all(flags_t mask) const;


  bool test_any(flags_t mask) const;


  void set(flags_t mask);


  void reset(flags_t mask);
};

namespace Parma_Polyhedra_Library {




  std::ostream& operator<<(std::ostream& s, const Status& u);





  std::istream& operator>>(std::istream& s, Status& u);
}
namespace Parma_Polyhedra_Library {

inline
Status::Status(flags_t mask)
  : flags(mask) {
}

inline
Status::Status()
  : flags(ZERO_DIM_UNIV) {
}

inline Status
operator&(const Status& x, const Status& y) {
  return Status(x.flags & y.flags);
}

inline Status
operator|(const Status& x, const Status& y) {
  return Status(x.flags | y.flags);
}

inline bool
Status::test_all(flags_t mask) const {
  return (flags & mask) == mask;
}

inline bool
Status::test_any(flags_t mask) const {
  return flags & mask;
}

inline void
Status::set(flags_t mask) {
  flags |= mask;
}

inline void
Status::reset(flags_t mask) {
  flags &= ~mask;
}

inline bool
Status::test_zero_dim_univ() const {
  return flags == ZERO_DIM_UNIV;
}

inline void
Status::reset_zero_dim_univ() {

  if (flags == ZERO_DIM_UNIV)

    flags = EMPTY;
}

inline void
Status::set_zero_dim_univ() {

  flags = ZERO_DIM_UNIV;
}

inline bool
Status::test_empty() const {
  return test_any(EMPTY);
}

inline void
Status::reset_empty() {
  reset(EMPTY);
}

inline void
Status::set_empty() {
  flags = EMPTY;
}

inline bool
Status::test_c_up_to_date() const {
  return test_any(C_UP_TO_DATE);
}

inline void
Status::reset_c_up_to_date() {
  reset(C_UP_TO_DATE);
}

inline void
Status::set_c_up_to_date() {
  set(C_UP_TO_DATE);
}

inline bool
Status::test_g_up_to_date() const {
  return test_any(G_UP_TO_DATE);
}

inline void
Status::reset_g_up_to_date() {
  reset(G_UP_TO_DATE);
}

inline void
Status::set_g_up_to_date() {
  set(G_UP_TO_DATE);
}

inline bool
Status::test_c_minimized() const {
  return test_any(C_MINIMIZED);
}

inline void
Status::reset_c_minimized() {
  reset(C_MINIMIZED);
}

inline void
Status::set_c_minimized() {
  set(C_MINIMIZED);
}

inline bool
Status::test_g_minimized() const {
  return test_any(G_MINIMIZED);
}

inline void
Status::reset_g_minimized() {
  reset(G_MINIMIZED);
}

inline void
Status::set_g_minimized() {
  set(G_MINIMIZED);
}

inline bool
Status::test_sat_c_up_to_date() const {
  return test_any(SAT_C_UP_TO_DATE);
}

inline void
Status::reset_sat_c_up_to_date() {
  reset(SAT_C_UP_TO_DATE);
}

inline void
Status::set_sat_c_up_to_date() {
  set(SAT_C_UP_TO_DATE);
}

inline bool
Status::test_sat_g_up_to_date() const {
  return test_any(SAT_G_UP_TO_DATE);
}

inline void
Status::reset_sat_g_up_to_date() {
  reset(SAT_G_UP_TO_DATE);
}

inline void
Status::set_sat_g_up_to_date() {
  set(SAT_G_UP_TO_DATE);
}

}
namespace Parma_Polyhedra_Library {
  bool operator==(const Polyhedron& x, const Polyhedron& y);
  bool operator!=(const Polyhedron& x, const Polyhedron& y);
  bool operator<(const Polyhedron& x, const Polyhedron& y);
  bool operator>(const Polyhedron& x, const Polyhedron& y);
  bool operator>=(const Polyhedron& x, const Polyhedron& y);


  bool operator<=(const Polyhedron& x, const Polyhedron& y);
  std::ostream& operator<<(std::ostream& s, const Polyhedron& p);
  std::istream& operator>>(std::istream& s, Polyhedron& p);

}
class Parma_Polyhedra_Library::Polyhedron {
public:

  enum Degenerate_Kind {

    UNIVERSE,

    EMPTY
  };

protected:

  Polyhedron(const Polyhedron& y);
  Polyhedron(Topology topol, size_t num_dimensions, Degenerate_Kind kind);
  Polyhedron(Topology topol, ConSys& cs);
  Polyhedron(Topology topol, GenSys& gs);
  template <class Box>
  Polyhedron(Topology topol, const Box& box);




  Polyhedron& operator=(const Polyhedron& y);

public:

  ~Polyhedron();


  size_t space_dimension() const;
  bool intersection_assign_and_minimize(const Polyhedron& y);
  void intersection_assign(const Polyhedron& y);
  bool poly_hull_assign_and_minimize(const Polyhedron& y);
  void poly_hull_assign(const Polyhedron& y);
  bool poly_difference_assign_and_minimize(const Polyhedron& y);
  void poly_difference_assign(const Polyhedron& y);
  Poly_Con_Relation relation_with(const Constraint& c) const;
  Poly_Gen_Relation relation_with(const Generator& g) const;
  void H79_widening_assign(const Polyhedron& y);
  void limited_H79_widening_assign(const Polyhedron& y, ConSys& cs);
  void time_elapse_assign(const Polyhedron& y);



  const ConSys& constraints() const;


  const ConSys& minimized_constraints() const;


  const GenSys& generators() const;


  const GenSys& minimized_generators() const;
  void add_constraint(const Constraint& c);
  void add_generator(const Generator& g);
  void affine_image(const Variable& var,
                    const LinExpression& expr,
                    const Integer& denominator = Integer_one());
  void affine_preimage(const Variable& var,
                       const LinExpression& expr,
                       const Integer& denominator = Integer_one());
  template <class Box>
  void shrink_bounding_box(Box& box) const;
  bool OK(bool check_not_empty = false) const;
  void add_dimensions_and_embed(size_t dim);
  void add_dimensions_and_project(size_t dim);
  void remove_dimensions(const std::set<Variable>& to_be_removed);
  void remove_higher_dimensions(size_t new_dimension);
  bool add_constraints_and_minimize(ConSys& cs);
  void add_constraints(ConSys& cs);
  void add_dimensions_and_constraints(ConSys& cs);
  bool add_generators_and_minimize(GenSys& gs);
  void add_generators(GenSys& gs);




  bool check_empty() const;




  bool check_universe() const;




  bool is_bounded() const;
  bool bounds_from_above(const LinExpression& expr) const;
  bool bounds_from_below(const LinExpression& expr) const;




  bool is_topologically_closed() const;


  void topological_closure_assign();
  friend bool
  Parma_Polyhedra_Library::operator<=(const Polyhedron& x,
                                      const Polyhedron& y);


  friend std::ostream&
  Parma_Polyhedra_Library::operator<<(std::ostream& s, const Polyhedron& p);


  friend std::istream&
  Parma_Polyhedra_Library::operator>>(std::istream& s, Polyhedron& p);
  void swap(Polyhedron& y);

private:

  ConSys con_sys;


  GenSys gen_sys;


  SatMatrix sat_c;


  SatMatrix sat_g;


  Status status;


  size_t space_dim;


  Topology topology() const;




  bool is_necessarily_closed() const;
  bool is_empty() const;


  bool constraints_are_up_to_date() const;


  bool generators_are_up_to_date() const;






  bool constraints_are_minimized() const;






  bool generators_are_minimized() const;




  bool sat_c_is_up_to_date() const;




  bool sat_g_is_up_to_date() const;
  void set_zero_dim_univ();




  void set_empty();


  void set_constraints_up_to_date();


  void set_generators_up_to_date();


  void set_constraints_minimized();


  void set_generators_minimized();


  void set_sat_c_up_to_date();


  void set_sat_g_up_to_date();







  void clear_empty();






  void clear_constraints_up_to_date();






  void clear_generators_up_to_date();


  void clear_constraints_minimized();


  void clear_generators_minimized();


  void clear_sat_c_up_to_date();


  void clear_sat_g_up_to_date();
  void update_constraints() const;
  bool update_generators() const;
  void update_sat_c() const;
  void update_sat_g() const;
  void obtain_sorted_constraints() const;
  void obtain_sorted_generators() const;
  void obtain_sorted_constraints_with_sat_c() const;
  void obtain_sorted_generators_with_sat_g() const;
  bool minimize() const;
  bool strongly_minimize() const;






  bool strongly_minimize_constraints() const;






  bool strongly_minimize_generators() const;
  bool bounds(const LinExpression& expr, bool from_above) const;
  static void add_dimensions(Matrix& mat1,
                             Matrix& mat2,
                             SatMatrix& sat1,
                             SatMatrix& sat2,
                             size_t add_dim);



  static size_t conversion(Matrix& entry,
                           size_t start,
                           Matrix& result,
                           SatMatrix& sat,
                           size_t num_lines_or_equalities);





  static int simplify(Matrix& mat, SatMatrix& sat);



  static bool minimize(bool con_to_gen, Matrix& source, Matrix& dest,
                       SatMatrix& sat);





  static bool add_and_minimize(bool con_to_gen,
                               Matrix& source1, Matrix& dest, SatMatrix& sat,
                               const Matrix& source2);






  void throw_topology_incompatible(const char* method,
                                   const Polyhedron& y) const;
  void throw_topology_incompatible(const char* method,
                                   const ConSys& ) const;
  void throw_topology_incompatible(const char* method,
                                   const Constraint& ) const;
  void throw_topology_incompatible(const char* method,
                                   const GenSys& ) const;
  void throw_topology_incompatible(const char* method,
                                   const Generator& ) const;

  void throw_dimension_incompatible(const char* method,
                                    const Polyhedron& y) const;
  void throw_dimension_incompatible(const char* method,
                                    const Matrix& y) const;
  void throw_dimension_incompatible(const char* method,
                                    const Row& y) const;
  void throw_dimension_incompatible(const char* method,
                                    size_t needed_dim) const;

  void throw_invalid_generator(const char* method) const;
  void throw_invalid_generators(const char* method) const;

  void throw_generic(const char* method, const char* reason) const;

  static void throw_topology_incompatible(const char* method,
                                          const Polyhedron& x,
                                          const Polyhedron& y);
  static void throw_dimension_incompatible(const char* method,
                                           const Polyhedron& x,
                                           const Polyhedron& y);


};


namespace std {



void swap(Parma_Polyhedra_Library::Polyhedron& x,
          Parma_Polyhedra_Library::Polyhedron& y);

}
namespace Parma_Polyhedra_Library {

inline Topology
Polyhedron::topology() const {


  return con_sys.topology();
}

inline void
Polyhedron::swap(Polyhedron& y) {
  if (topology() != y.topology())
    throw_topology_incompatible("swap(y)", y);
  std::swap(con_sys, y.con_sys);
  std::swap(gen_sys, y.gen_sys);
  std::swap(sat_c, y.sat_c);
  std::swap(sat_g, y.sat_g);
  std::swap(status, y.status);
  std::swap(space_dim, y.space_dim);
}


inline bool
Polyhedron::is_necessarily_closed() const {


  return con_sys.is_necessarily_closed();
}

inline bool
Polyhedron::is_empty() const {
  return status.test_empty();
}

inline bool
Polyhedron::constraints_are_up_to_date() const {
  return status.test_c_up_to_date();
}

inline bool
Polyhedron::generators_are_up_to_date() const {
  return status.test_g_up_to_date();
}

inline bool
Polyhedron::constraints_are_minimized() const {
  return status.test_c_minimized();
}

inline bool
Polyhedron::generators_are_minimized() const {
  return status.test_g_minimized();
}

inline bool
Polyhedron::sat_c_is_up_to_date() const {
  return status.test_sat_c_up_to_date();
}

inline bool
Polyhedron::sat_g_is_up_to_date() const {
  return status.test_sat_g_up_to_date();
}

inline void
Polyhedron::set_constraints_up_to_date() {
  status.set_c_up_to_date();
}

inline void
Polyhedron::set_generators_up_to_date() {
  status.set_g_up_to_date();
}

inline void
Polyhedron::set_constraints_minimized() {
  set_constraints_up_to_date();
  status.set_c_minimized();
}

inline void
Polyhedron::set_generators_minimized() {
  set_generators_up_to_date();
  status.set_g_minimized();
}

inline void
Polyhedron::set_sat_c_up_to_date() {
  status.set_sat_c_up_to_date();
}

inline void
Polyhedron::set_sat_g_up_to_date() {
  status.set_sat_g_up_to_date();
}

inline void
Polyhedron::clear_empty() {
  status.reset_empty();
}

inline void
Polyhedron::clear_sat_c_up_to_date() {
  status.reset_sat_c_up_to_date();

}

inline void
Polyhedron::clear_sat_g_up_to_date() {
  status.reset_sat_g_up_to_date();

}

inline void
Polyhedron::clear_constraints_minimized() {
  status.reset_c_minimized();
}

inline void
Polyhedron::clear_generators_minimized() {
  status.reset_g_minimized();
}

inline void
Polyhedron::clear_constraints_up_to_date() {
  clear_constraints_minimized();
  clear_sat_c_up_to_date();
  clear_sat_g_up_to_date();
  status.reset_c_up_to_date();

}

inline void
Polyhedron::clear_generators_up_to_date() {
  clear_generators_minimized();
  clear_sat_c_up_to_date();
  clear_sat_g_up_to_date();
  status.reset_g_up_to_date();

}


inline bool
operator==(const Polyhedron& x, const Polyhedron& y) {
  return x <= y && y <= x;
}


inline bool
operator!=(const Polyhedron& x, const Polyhedron& y) {
  return !(x == y);
}


inline bool
operator>=(const Polyhedron& x, const Polyhedron& y) {
  return y <= x;
}


inline bool
operator<(const Polyhedron& x, const Polyhedron& y) {
  return x <= y && !(x >= y);
}


inline bool
operator>(const Polyhedron& x, const Polyhedron& y) {
  return y < x;
}


inline size_t
Polyhedron::space_dimension() const {
  return space_dim;
}

inline bool
Polyhedron::check_empty() const {
  minimize();
  return is_empty();
}

inline bool
Polyhedron::bounds_from_above(const LinExpression& expr) const {
  return bounds(expr, true);
}

inline bool
Polyhedron::bounds_from_below(const LinExpression& expr) const {
  return bounds(expr, false);
}

template <class Box>
Polyhedron::Polyhedron(Topology topol, const Box& box)
  : con_sys(topol),
    gen_sys(topol),
    sat_c(),
    sat_g() {

  space_dim = box.space_dimension();


  if (box.is_empty()) {
    set_empty();
    return;
  }


  if (space_dim == 0) {
    set_zero_dim_univ();
    return;
  }




  con_sys.insert(Variable(space_dim - 1) >= 0);

  for (size_t k = space_dim; k-- > 0; ) {

    bool l_closed = false;
    Integer l_n, l_d;
    bool l_bounded = box.get_lower_bound(k, l_closed, l_n, l_d);
    if (l_bounded && topol == NECESSARILY_CLOSED && !l_closed)
      throw_generic("C_Polyhedron(const Box& box)",
                    "box has an open lower bound");

    bool u_closed = false;
    Integer u_n, u_d;
    bool u_bounded = box.get_upper_bound(k, u_closed, u_n, u_d);
    if (u_bounded && topol == NECESSARILY_CLOSED && !u_closed)
      throw_generic("C_Polyhedron(const Box& box)",
                    "box has an open upper bound");


    if (l_bounded && u_bounded
        && l_closed && u_closed
        && l_n == u_n && l_d == u_d) {

      con_sys.insert(l_d * Variable(k) == l_n);
    }
    else {

      if (l_bounded) {
       if (l_closed)

         con_sys.insert(l_d * Variable(k) >= l_n);
       else

         con_sys.insert(l_d * Variable(k) > l_n);
      }

      if (u_bounded) {
       if (u_closed)

         con_sys.insert(u_d * Variable(k) <= u_n);
       else

         con_sys.insert(u_d * Variable(k) < u_n);
      }
    }
  }

  if (topol == NECESSARILY_CLOSED)

    con_sys.insert(Constraint::zero_dim_positivity());
  else {


    con_sys.insert(Constraint::epsilon_leq_one());
    con_sys.insert(Constraint::epsilon_geq_zero());
  }


  size_t n_rows = con_sys.num_rows() - 1;
  con_sys[0].swap(con_sys[n_rows]);
  con_sys.erase_to_end(n_rows);


  set_constraints_up_to_date();
  assert(OK());
}

template <class Box>
void
Polyhedron::shrink_bounding_box(Box& box) const {
  if (check_universe())
    return;

  if (check_empty()) {
    box.set_empty();
    return;
  }

  if (space_dim == 0)
    return;


  std::vector<LBoundary> lower_bound(space_dim);
  std::vector<UBoundary> upper_bound(space_dim);

  for (size_t j = space_dim; j-- > 0; ) {

    lower_bound[j] = LBoundary(ExtendedRational('+'), LBoundary::OPEN);

    upper_bound[j] = UBoundary(ExtendedRational('-'), UBoundary::OPEN);
  }


  const GenSys& gs = generators();
  const GenSys::const_iterator gs_begin = gs.begin();
  const GenSys::const_iterator gs_end = gs.end();



  for (GenSys::const_iterator i = gs_begin; i != gs_end; ++i) {
    const Generator& g = *i;
    Generator::Type g_type = g.type();
    switch (g_type) {
    case Generator::LINE:


      for (size_t j = space_dim; j-- > 0; )
        if (g.coefficient(Variable(j)) != 0) {
          lower_bound[j] = LBoundary(ExtendedRational('-'), LBoundary::OPEN);
          upper_bound[j] = UBoundary(ExtendedRational('+'), UBoundary::OPEN);
        }
    break;
    case Generator::RAY:


      for (size_t j = space_dim; j-- > 0; ) {
        int sign = sgn(g.coefficient(Variable(j)));
        if (sign < 0)
          lower_bound[j] = LBoundary(ExtendedRational('-'), LBoundary::OPEN);
        else if (sign > 0)
          upper_bound[j] = UBoundary(ExtendedRational('+'), UBoundary::OPEN);
      }
      break;
    case Generator::POINT:
    case Generator::CLOSURE_POINT:
      {
        const Integer& d = g.divisor();
        for (size_t j = space_dim; j-- > 0; ) {
          const Integer& n = g.coefficient(Variable(j));
          ExtendedRational r(n, d);
          LBoundary lb(r,(g_type == Generator::CLOSURE_POINT
                          ? LBoundary::OPEN
                          : LBoundary::CLOSED));
          if (lb < lower_bound[j])
            lower_bound[j] = lb;
          UBoundary ub(r, (g_type == Generator::CLOSURE_POINT
                           ? UBoundary::OPEN
                           : UBoundary::CLOSED));
          if (ub > upper_bound[j])
            upper_bound[j] = ub;
        }
      }
      break;
    }
  }


  for (size_t j = space_dim; j-- > 0; ) {

    const LBoundary& lb = lower_bound[j];
    const ExtendedRational& lr = lb.bound();
    if (lr.direction_of_infinity() == 0)
      box.raise_lower_bound(j, lb.is_closed(),
                            lr.numerator(), lr.denominator());


    const UBoundary& ub = upper_bound[j];
    const ExtendedRational& ur = ub.bound();
    if (ur.direction_of_infinity() == 0)
      box.lower_upper_bound(j, ub.is_closed(),
                            ur.numerator(), ur.denominator());
  }
}

}



inline void
std::swap(Parma_Polyhedra_Library::Polyhedron& x,
          Parma_Polyhedra_Library::Polyhedron& y) {
  x.swap(y);
}
class Parma_Polyhedra_Library::C_Polyhedron : public Polyhedron {
public:
  explicit C_Polyhedron(size_t num_dimensions = 0,
                        Degenerate_Kind kind = UNIVERSE);
  C_Polyhedron(ConSys& cs);
  C_Polyhedron(GenSys& gs);






  explicit C_Polyhedron(const NNC_Polyhedron& y);
  template <class Box>
  C_Polyhedron(const Box& box, From_Bounding_Box dummy);


  C_Polyhedron(const C_Polyhedron& y);




  C_Polyhedron& operator=(const C_Polyhedron& y);


  ~C_Polyhedron();
};
namespace Parma_Polyhedra_Library {

inline
C_Polyhedron::C_Polyhedron(size_t num_dimensions, Degenerate_Kind kind)
  : Polyhedron(NECESSARILY_CLOSED, num_dimensions, kind) {
}

inline
C_Polyhedron::C_Polyhedron(ConSys& cs)
  : Polyhedron(NECESSARILY_CLOSED, cs) {
}

inline
C_Polyhedron::C_Polyhedron(GenSys& gs)
  : Polyhedron(NECESSARILY_CLOSED, gs) {
}

template <class Box>
C_Polyhedron::C_Polyhedron(const Box& box, From_Bounding_Box)
  : Polyhedron(NECESSARILY_CLOSED, box) {
}

inline
C_Polyhedron::C_Polyhedron(const C_Polyhedron& y)
  : Polyhedron(y) {
}

inline C_Polyhedron&
C_Polyhedron::operator=(const C_Polyhedron& y) {
  Polyhedron::operator=(y);
  return *this;
}

inline
C_Polyhedron::~C_Polyhedron() {
}

}
class Parma_Polyhedra_Library::NNC_Polyhedron : public Polyhedron {
public:
  explicit NNC_Polyhedron(size_t num_dimensions = 0,
                          Degenerate_Kind kind = UNIVERSE);
  NNC_Polyhedron(ConSys& cs);
  NNC_Polyhedron(GenSys& gs);


  explicit NNC_Polyhedron(const C_Polyhedron& y);
  template <class Box>
  NNC_Polyhedron(const Box& box, From_Bounding_Box dummy);


  NNC_Polyhedron(const NNC_Polyhedron& y);




  NNC_Polyhedron& operator=(const NNC_Polyhedron& y);


  ~NNC_Polyhedron();

};
namespace Parma_Polyhedra_Library {

inline
NNC_Polyhedron::NNC_Polyhedron(size_t num_dimensions, Degenerate_Kind kind)
  : Polyhedron(NOT_NECESSARILY_CLOSED, num_dimensions, kind) {
}

inline
NNC_Polyhedron::NNC_Polyhedron(ConSys& cs)
  : Polyhedron(NOT_NECESSARILY_CLOSED, cs) {
}

inline
NNC_Polyhedron::NNC_Polyhedron(GenSys& gs)
  : Polyhedron(NOT_NECESSARILY_CLOSED, gs) {
}

template <class Box>
NNC_Polyhedron::NNC_Polyhedron(const Box& box, From_Bounding_Box)
  : Polyhedron(NOT_NECESSARILY_CLOSED, box) {
}

inline
NNC_Polyhedron::NNC_Polyhedron(const NNC_Polyhedron& y)
  : Polyhedron(y) {
}

inline NNC_Polyhedron&
NNC_Polyhedron::operator=(const NNC_Polyhedron& y) {
  Polyhedron::operator=(y);
  return *this;
}

inline
NNC_Polyhedron::~NNC_Polyhedron() {
}

}

#ifdef PPL_SAVE_NDEBUG
#define NDEBUG 1
#undef PPL_SAVE_NDEBUG
#endif

#undef PPL_FLEXIBLE_ARRAY
#undef PACKAGE
#undef PACKAGE_BUGREPORT
#undef PACKAGE_NAME
#undef PACKAGE_STRING
#undef PACKAGE_TARNAME
#undef PACKAGE_VERSION
#undef VERSION

#endif
